/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.util.BitSet;

import fileselection.calculation.CalculationParser;
import fileselection.calculation.CalculationTree;


/**
 * A select list which performs calculations to obtain its set of
 * files which belong to it.  The select list is considered read-only
 * from the point of view of the user.
 */
public class CalculationSelectList extends SelectList
{
	/**
	 * The bit set indicating which files belong to this select list.
	 * The index of a bit is the index value for the file.
	 * This is null if the bit set is not known or is invalidated,
	 * and is then allocated with the correct values on demand.
	 */
	protected BitSet mBitSet;

	/**
	 * The calculation node tree to be evaluated for generating our
	 * bit set.  This is null if our expression has not yet been
	 * parsed successfully.
	 */
	protected CalculationTree mCalculationTree;


	/**
	 * Constructor which accepts the owning file selection manager,
	 * the select list name, and the description of this select list.
	 * For us the description is an expression to be evaluated.
	 */
	public CalculationSelectList(FileSelectionManager manager, String name,
		String description)
	{
		super(manager, name, description);
	}


	/**
	 * Parse the description text as an expression to be evaluated and
	 * save the resulting CalculationNode tree for use during evaluation.
	 * Returns true if the expression was parsed correctly.
	 */
	public boolean parseDescriptionExpression()
	{
		//
		// Get the parser and use it on our description string.
		//
		CalculationParser parser = new CalculationParser(mManager);

		CalculationTree tree = parser.parse(mDescription, false);

		//
		// If the parse failed then return failure.
		//
		if (tree == null)
			return false;

		//
		// Save the resulting node tree and return success.
		//
		mCalculationTree = tree;

		return true;
	}


	/**
	 * Evaluate the compiled expression and save the resulting BitSet
	 * as the set of files in our select list.  Returns true on success.
	 */
	public boolean evaluateExpression()
	{
		//
		// If the expression hasn't been parsed yet then fail.
		//
		if (mCalculationTree == null)
			return false;

		//
		// Evaluate the compiled expression into a bit set (which might
		// fail and return null which is ok).
		//
		mBitSet = mCalculationTree.evaluateBitSet();

		//
		// Update our last modified change counter.
		//
		updateModifiedChangeCounter();

		//
		// Return success if a bit set was produced.
		//
		return (mBitSet != null);
	}


	/**
	 * Return the type of select list.
	 */
	public String getType()
	{
		return "Calculation";
	}


	/**
	 * Return whether or not the file with the specifed index is in
	 * this select list.
	 */
	public boolean isFileInList(int index)
	{
		if ((mBitSet == null) && !evaluateExpression())
			return false;

		if ((index < 0) || (index >= mBitSet.length()))
			return false;

		return mBitSet.get(index);
	}


	/**
	 * Return whether or not the file having the index contained within a
	 * FileInfo is in this select list.
	 */
	public boolean isFileInList(FileInfo fileInfo)
	{
		if (fileInfo == null)
			return false;

		return isFileInList(fileInfo.getIndex());
	}


	/**
	 * Return a file bit set for the files selected by this select list.
	 * Warning: Modifying the bit set will affect our select list.
	 */
	public BitSet getFileBitSet()
	{
		if ((mBitSet == null) && !evaluateExpression())
			return new BitSet();

		return mBitSet;
	}


	/**
	 * Return the count of files which belong to this select list.
	 */
	public int getFileCount()
	{
		if ((mBitSet == null) && !evaluateExpression())
			return 0;

		return mBitSet.cardinality();
	}
}
