/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;


/**
 * A class to build and run a command line panel for the main window.
 * This contains a text field for entering commands, a text field for
 * displaying results of the commands, and labels identifying the fields.
 * The actual command execution is done using the Commands class.
 */
public class CommandPanel extends JPanel
	implements ActionListener, KeyListener
{
	/**
	 * A text field for the user to enter commands.
	 */
	protected JTextField mCommandTextField;

	/**
	 * A text field for results to be displayed.
	 */
	protected JTextField mResultTextField;

	/**
	 * The command executing class.
	 */
	protected Commands mCommands;

	/**
	 * The current history index.
	 * A value of -1 indicates that we are at the end.
	 */
	protected int mHistoryIndex = -1;

	/**
	 * The color to be used for error responses.
	 */
	protected Color mErrorColor;


	/**
	 * Constructor which accepts the main window.
	 */
	public CommandPanel(MainWindow mainWindow)
	{
		super(new BorderLayout());

		//
		// Allocate the command class.
		//
		mCommands = new Commands(mainWindow.getManager());

		//
		// Build a subpanel containing the two labels arranged vertically
		// with right alignment.
		//
		JPanel labelSubPanel = new JPanel(new GridLayout(0, 1));

		JLabel commandLabel = new JLabel("Command: ");
		JLabel resultLabel = new JLabel("Result: ");

		commandLabel.setHorizontalAlignment(SwingConstants.RIGHT);
		resultLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		labelSubPanel.add(commandLabel);
		labelSubPanel.add(resultLabel);

		//
		// Build a subpanel containing the two text fields arranged vertically.
		// The command text field is editable, but the result one is not.
		//
		JPanel commandSubPanel = new JPanel(new GridLayout(0, 1));

		mCommandTextField = new JTextField(80);
		mResultTextField = new JTextField(80);

		mResultTextField.setEditable(false);

		commandSubPanel.add(mCommandTextField);
		commandSubPanel.add(mResultTextField);

		//
		// Add an action listener to the command text field so we can
		// execute its commands when the enter key is typed.
		//
		mCommandTextField.addActionListener(this);

		//
		// Add a key event listener so that we can scan up and down through
		// the list of previously executed commands.
		//
		mCommandTextField.addKeyListener(this);

		//
		// Add the labels to the left of the this panel and the text
		// fields to the remainder of this panel.
		//
		add(labelSubPanel, BorderLayout.WEST);
		add(commandSubPanel, BorderLayout.CENTER);

		//
		// Allocate the color to use for color responses.
		// This is a dark red to show up better against the background.
		//
		mErrorColor = Color.RED.darker();
	}


	/**
	 * Implementation of ActionListener method when the command is
	 * complete in our command text field.
	 */
	public void actionPerformed(ActionEvent e)
	{
		//
		// Make sure the source is the one we expect.
		//
		if (e.getSource() != mCommandTextField)
			return;

		//
		// Set the response blank while the command executes.
		//
		mResultTextField.setText("");

		//
		// Get the command to be executed.
		//
		String commandLine = mCommandTextField.getText();

		//
		// Execute the command and get the success flag and response line.
		//
		boolean success = mCommands.execute(commandLine);

		String responseLine = mCommands.getResponseLine();

		//
		// Set the color of the response red if the command failed,
		// and then set the response text and clear the command field.
		//
		mResultTextField.setForeground(success ? Color.black : mErrorColor);
		mResultTextField.setText(responseLine);

		mCommandTextField.setText("");

		mHistoryIndex = -1;
	}


	/**
	 * Implementation of KeyListener method called when a key is pressed.
	 * We look for the up, down, page up, and page down keys so as to
	 * move around in the command history.
	 */
	public void keyPressed(KeyEvent e)
	{
		//
		// Initialize the actions we might perform.
		//
		boolean saveCommand = false;
		boolean loadCommand = false;
		boolean clearCommand = false;
		boolean clearResult = false;

		//
		// Get some values in case we need them.
		//
		int historySize = mCommands.getCommandLineHistorySize();
		String currentCommand = mCommandTextField.getText();

		//
		// Get the key code that was pressed and switch on it.
		//
		int code = e.getKeyCode();

		switch (code)
		{
		case KeyEvent.VK_UP:
			//
			// We want to move up to older commands in the history.
			//
			clearResult = true;

			if (mHistoryIndex == 0)
				break;

			if (mHistoryIndex < 0)
			{
				saveCommand = true;
				mHistoryIndex = historySize - 1;
			}
			else
				mHistoryIndex--;

			if (mHistoryIndex >= 0)
				loadCommand = true;

			break;

		case KeyEvent.VK_DOWN:
			//
			// We want to move down to newer commands in the history.
			//
			clearResult = true;

			if ((mHistoryIndex < 0) || (mHistoryIndex + 1 >= historySize))
			{
				saveCommand = true;
				clearCommand = true;
				mHistoryIndex = -1;

				break;
			}

			mHistoryIndex++;

			loadCommand = true;

			break;

		case KeyEvent.VK_PAGE_UP:
			//
			// We want to move to the top oldest command of the history.
			//
			clearResult = true;

			if (mHistoryIndex < 0)
				saveCommand = true;

			if (historySize > 0)
			{
				mHistoryIndex = 0;
				loadCommand = true;
			}

			break;

		case KeyEvent.VK_PAGE_DOWN:
			//
			// We want to move to the bottom newest command of the history.
			//
			if (mHistoryIndex < 0)
				saveCommand = true;

			clearResult = true;
			clearCommand = true;
			mHistoryIndex = -1;

			break;
		}

		//
		// Now perform the updating of the text fields as calculated.
		//
		if (loadCommand)
		{
			String command = mCommands.getCommandLineHistory(mHistoryIndex);
			mCommandTextField.setText(command);
		}

		if (saveCommand)
			mCommands.addCommandLineHistory(currentCommand);

		if (clearCommand)
		{
			mCommandTextField.setText("");
			mHistoryIndex = -1;
		}

		if (clearResult)
			mResultTextField.setText("");
	}


	/**
	 * Implementation of KeyListener method called when a key is released.
	 * We ignore these.
	 */
	public void keyReleased(KeyEvent e)
	{
	}


	/**
	 * Implementation of KeyListener method called when a key is typed.
	 * We ignore these.
	 */
	public void keyTyped(KeyEvent e)
	{
	}
}
