/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.util.ArrayList;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;

import fileselection.dialog.AddFileToSelectListDialog;
import fileselection.dialog.RemoveFileFromSelectListDialog;
import fileselection.operation.RunCommandOnFileOperation;

import fileselection.utilities.Utils;
import fileselection.utilities.Settings;


/**
 * A class to process some common actions for a specified file.
 * The actions are invoked either from a pop-up menu when right-clicking
 * on a row in the file table or by clicking on buttons in the interactive
 * file dialog.  This class also keeps track of a set of manual select
 * lists which have been defined as deserving their own menu items in a
 * file actions menu.
 */
public class FileActions
{
	/**
	 * The current file being processed.
	 */
	protected FileInfo mCurrentFile;

	/**
	 * Our file selection manager.
	 */
	protected FileSelectionManager mManager;

	/**
	 * The main window.
	 */
	protected MainWindow mMainWindow;

	/**
	 * A list of select lists which have their own buttons for adding.
	 */
	protected ArrayList mAddToSelectListArrayList;

	/**
	 * A list of select lists which have their own buttons for removing.
	 */
	protected ArrayList mRemoveFromSelectListArrayList;

	/**
	 * Our parent component.
	 */
	protected Component mParent;

	/**
	 * The settings class.
	 */
	protected Settings mSettings;


	/**
	 * Constructor which sets the main window.
	 */
	public FileActions(MainWindow mainWindow)
	{
		//
		// Extract the values we need.
		//
		mMainWindow = mainWindow;
		mManager = mainWindow.getManager();
		mParent = mainWindow.getJFrame();

		//
		// Get the settings for our defaults.
		//
		mSettings = Settings.getInstance();
	}


	/**
	 * Set the parent component for our popup dialogs.
	 */
	public void setParent(Component parent)
	{
		mParent = parent;
	}


	/**
	 * Set the current file which all of our actions will act on.
	 */
	public void setCurrentFile(FileInfo currentFile)
	{
		mCurrentFile = currentFile;
	}


	/**
	 * Return an array of the select lists which have been set as
	 * desiring their own menu item for adding files.  Returns an empty
	 * array if there are none.
	 */
	public ManualSelectList[] getAddToSelectListArray()
	{
		if (mAddToSelectListArrayList == null)
			return new ManualSelectList[0];

		int size = mAddToSelectListArrayList.size();

		ManualSelectList[] array = new ManualSelectList[size];

		mAddToSelectListArrayList.toArray(array);

		return array;
	}


	/**
	 * Return an array of the select lists which have been set as
	 * desiring their own menu item for removing files.  Returns an
	 * empty array if there are none.
	 */
	public ManualSelectList[] getRemoveFromSelectListArray()
	{
		if (mRemoveFromSelectListArrayList == null)
			return new ManualSelectList[0];

		int size = mRemoveFromSelectListArrayList.size();

		ManualSelectList[] array = new ManualSelectList[size];

		mRemoveFromSelectListArrayList.toArray(array);

		return array;
	}


	/**
	 * Clear the "is changed" status for the current file.
	 */
	public void clearIsChangedAction()
	{
		if (mCurrentFile != null)
			mCurrentFile.clearIsChanged();
	}


	/**
	 * Open the file using an associated external program.
	 * This can also open directories.
	 */
	public void openFileAction()
	{
		if (mCurrentFile == null)
		{
			showOpenFileWarning("No current file");

			return;
		}

		//
		// Get the full file name and the extension of the current file.
		//
		String fileName = mCurrentFile.getFileName();
		String extension = mCurrentFile.getExtension();

		//
		// Get the file association manager.
		//
		FileAssociation association = mManager.getFileAssociation();

		//
		// If the file is a directory then we always open it with the
		// default command defined for that purpose.
		//
		if (mCurrentFile.isDirectory())
		{
			//
			// Get the edit command for directories.
			//
			String editCommand = association.getDefaultDirectoryEditCommand();

			//
			// If there is no command then complain.
			//
			if (editCommand == null)
			{
				showOpenFileWarning("No edit command is defined for directories");

				return;
			}

			//
			// Run the command on the file name.
			//
			runCommand(editCommand, fileName);

			return;
		}

		//
		// This is a normal file so the command depends on the extension.
		// Get the edit command defined for this file extension.
		//
		String editCommand = association.getEditCommand(extension);

		//
		// If no edit command is defined then complain.
		//
		if (editCommand == null)
		{
			showOpenFileWarning("No edit command associated with extension \"" +
				extension + "\"");

			return;
		}

		//
		// Run the command on the file name.
		//
		runCommand(editCommand, fileName);
	}


	/**
	 * Run an external program on the current file.
	 */
	public void runProgramOnFileAction()
	{
		if (mCurrentFile == null)
		{
			showOpenFileWarning("No current file");

			return;
		}

		//
		// Build the message to ask for a command.
		//
		JPanel messagePanel = new JPanel(new BorderLayout());
		JTextField commandField = new JTextField(60);
		messagePanel.add(new JLabel("Command: "), BorderLayout.WEST);
		messagePanel.add(commandField, BorderLayout.CENTER);

		//
		// Show the dialog to enter the command.
		//
		int result = JOptionPane.showConfirmDialog(mParent, messagePanel,
			"Run Program On File", JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return;

		//
		// Get the edit command.
		//
		String editCommand = Utils.trimSpaces(commandField.getText());

		//
		// Run the specified command.
		//
		runCommand(editCommand, mCurrentFile.getFileName());
	}


	/**
	 * Run the specified command on the specified file name.
	 */
	public void runCommand(String command, String fileName)
	{
		//
		// Check the arguments.
		//
		if ((command == null) || (command.length() == 0))
		{
			showOpenFileWarning("No command given to operate on file");

			return;
		}

		if ((fileName == null) || (fileName.length() == 0))
		{
			showOpenFileWarning("No file name for command");

			return;
		}

		//
		// Get the operation which runs commands on files.
		//
		RunCommandOnFileOperation operation =
			new RunCommandOnFileOperation(mManager);

		//
		// Configure the operation.
		//
		operation.setCommand(command);
		operation.setFileName(fileName);
		operation.setDefaultReplacementString();

		//
		// Try to make the operation ready for execution.
		//
		if (!operation.isReadyForExecution())
		{
			showOpenFileWarning("Run program setup error: " +
				operation.getErrorMessage());

			return;
		}

		//
		// Run the command in a background thread.
		//
		if (!operation.startOperation())
		{
			showOpenFileWarning("Run program start error: " +
				operation.getErrorMessage());

			return;	
		}
	}


	/**
	 * Update the file status.
	 */
	public void updateFileStatusAction()
	{
		if (mCurrentFile != null)
			mCurrentFile.updateStatus();
	}


	/**
	 * Add the file to a chosen select list.
	 * If there is one select list chosen and the add select list
	 * to menu check box was checked, then this will add the select
	 * list to a list of adding select lists for future use.
	 * Returns the select list that was added, or null.
	 */
	public ManualSelectList addToSelectListAction()
	{
		//
		// If there is no current file then complain.
		//
		if (mCurrentFile == null)
		{
			showOpenFileWarning("No current file");

			return null;
		}

		//
		// Run the add file to select list dialog to get the list of
		// select lists to be added to.
		//
		AddFileToSelectListDialog dialog =
			new AddFileToSelectListDialog(mMainWindow, mParent);

		if (!dialog.show())
			return null;

		//
		// Get the select list array.
		//
		ManualSelectList[] selectLists = dialog.getManualSelectListArray();

		//
		// Add the file to each of the select lists.
		//
		int fileIndex = mCurrentFile.getIndex();

		for (int index = 0; index < selectLists.length; index++)
			selectLists[index].addFileToList(fileIndex);

		//
		// If the create button check box was not set or if exactly
		// one select list was not chosen, then we are done.
		//
		if (!dialog.isCreateButtonWanted() || (selectLists.length != 1))
			return null;

		//
		// We want to add the chosen select list to the add list.
		// Allocate the adding array list if necessary.
		//
		if (mAddToSelectListArrayList == null)
			mAddToSelectListArrayList = new ArrayList(10);

		//
		// If the select list is already present then do nothing.
		//
		ManualSelectList selectList = selectLists[0];

		if (mAddToSelectListArrayList.contains(selectList))
			return null;

		//
		// Add the select list to the adding list and return it.
		//
		mAddToSelectListArrayList.add(selectList);

		return selectList;
	}


	/**
	 * Remove the file from a chosen select list.
	 * If there is one select list chosen and the add select list
	 * to menu check box was checked, then this will add the select
	 * list to a list of removing select lists for future use.
	 * Returns the select list that was added, or null.
	 */
	public ManualSelectList removeFromSelectListAction()
	{
		//
		// If there is no current file then complain.
		//
		if (mCurrentFile == null)
		{
			showOpenFileWarning("No current file");

			return null;
		}

		//
		// Run the remove file from select list dialog to get the list of
		// select lists to be removed from.
		//
		RemoveFileFromSelectListDialog dialog =
			new RemoveFileFromSelectListDialog(mMainWindow, mParent);

		if (!dialog.show())
			return null;

		//
		// Get the select list array.
		//
		ManualSelectList[] selectLists = dialog.getManualSelectListArray();

		//
		// Remove the file from each of the select lists.
		//
		int fileIndex = mCurrentFile.getIndex();

		for (int index = 0; index < selectLists.length; index++)
			selectLists[index].removeFileFromList(fileIndex);

		//
		// If the create button check box was not set or if exactly
		// one select list was not chosen, then we are done.
		//
		if (!dialog.isCreateButtonWanted() || (selectLists.length != 1))
			return null;

		//
		// We want to add the chosen select list to the remove list.
		// Allocate the removing array list if necessary.
		//
		if (mRemoveFromSelectListArrayList == null)
			mRemoveFromSelectListArrayList = new ArrayList(10);

		//
		// If the select list is already present then do nothing.
		//
		ManualSelectList selectList = selectLists[0];

		if (mRemoveFromSelectListArrayList.contains(selectList))
			return null;

		//
		// Add the select list to the removing list and return it.
		//
		mRemoveFromSelectListArrayList.add(selectList);

		return selectList;
	}


	/**
	 * Add the file to the manual select list indicated by the given index.
	 * This is the index into the optional adding select list.
	 */
	public void addFileToSelectListAction(int index)
	{
		//
		// If there is no current file then complain.
		//
		if (mCurrentFile == null)
		{
			showOpenFileWarning("No current file");

			return;
		}

		//
		// Range check the index.
		//
		if (mAddToSelectListArrayList == null)
			return;

		if ((index < 0) || (index >= mAddToSelectListArrayList.size()))
			return;

		//
		// Get the indicated select list and add the file to it.
		//
		ManualSelectList selectList =
			(ManualSelectList) mAddToSelectListArrayList.get(index);

		selectList.addFileToList(mCurrentFile.getIndex());
	}


	/**
	 * Remove the file from the manual select list indicated by the given index.
	 * This is the index into the optional removing select list.
	 */
	public void removeFileFromSelectListAction(int index)
	{
		//
		// If there is no current file then complain.
		//
		if (mCurrentFile == null)
		{
			showOpenFileWarning("No current file");

			return;
		}

		//
		// Range check the index.
		//
		if (mRemoveFromSelectListArrayList == null)
			return;

		if ((index < 0) || (index >= mRemoveFromSelectListArrayList.size()))
			return;

		//
		// Get the indicated select list and remove the file to it.
		//
		ManualSelectList selectList =
			(ManualSelectList) mRemoveFromSelectListArrayList.get(index);

		selectList.removeFileFromList(mCurrentFile.getIndex());
	}


	/**
	 * Show a warning dialog about opening a file.
	 */
	public void showOpenFileWarning(String message)
	{
		showWarning("Open File", message);
	}


	/**
	 * Show a warning dialog with the indicated title and message.
	 */
	public void showWarning(String title, String message)
	{
		JOptionPane.showMessageDialog(mParent, message, title,
			JOptionPane.WARNING_MESSAGE);
	}
}
