/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import fileselection.utilities.ArrayAndMap;
import fileselection.utilities.ChangeCounter;


/**
 * A table model for the FileInfo table.
 * Each row shows a file name, the file's properties, and the select
 * lists which the file belongs to, all of which is configurable.
 * The row can also be color coded to indicate select list inclusion.
 * The files which are displayed can be filtered to reduce the number shown.
 * The rows can be sorted by any of the columns.
 *
 * Note: Unlike most other table models, updates to the rows of the model
 * do not immediately fire table model events.  Instead they are remembered
 * in the change counter variables both for the individual rows and for the
 * table as a whole.  The deferred table model events are fired by calling
 * firePendingRowUpdates.  This causes a performance enhancement when
 * adding lots of rows, avoids threading problems between the swing and
 * background processes, and makes it easier to track interrelated changes
 * between the file info table and the select table.
 */
public class FileInfoTableModel extends AbstractTableModel
	implements ChangeCounter
{
	/**
	 * The use value for select lists which are used as columns.
	 */
	public static final String COLUMN_USE = "column";

	/**
	 * The preferred width of a select list column.
	 * This size is sufficient to show a radio button and a small
	 * heading string.
	 */
	public static final int SELECT_LIST_COLUMN_WIDTH = 20;

	/**
	 * The special use value for the filtering select list.
	 */
	public static final String FILTER_USE = "filter";

	/**
	 * Our owning file selection manager.
	 */
	protected FileSelectionManager mManager;

	/**
	 * The select list which filters the displayed files.
	 * If this is null then no filtering is done.
	 */
	protected SelectList mSelectListFilter;

	/**
	 * Whether a select list filter shows its selected files.
	 * (If not, then the select list hides its selected files instead.)
	 */
	protected boolean mSelectListFilterShowsFiles;

	/**
	 * The full list of files that might be displayed in the file table.
	 * The actual files which are displayed can be a subset of this.
	 */
	protected ArrayAndMap mFiles;

	/**
	 * The filtered list of rows for the table which are displayed.
	 * If there is no filter select list then this is the same array
	 * as stored in mFiles.  Each row is a FileInfo object.
	 */
	protected ArrayList mFilteredFileList;

	/**
	 * The list of columns for the table indexed by the column number.
	 * Each column is either a FileInfoItem or a SelectList class.
	 * A FileInfoItem column displays file information as specified by
	 * its value.  A SelectList column displays whether or not the file
	 * belongs to the specified select list.
	 */
	protected ArrayList mColumnList = new ArrayList(20);

	/**
	 * An array of column widths for the columns.
	 */
	protected ArrayList mColumnWidthList = new ArrayList(20);

	/**
	 * The last modified change counter for this table model.
	 * Every change to the table stores the newest change counter value.
	 * This is used by select lists to update themselves.
	 */
	protected long mModifiedChangeCounter;

	/**
	 * The displayed change counter value for this table model.
	 */
	protected long mDisplayedChangeCounter;


	/**
	 * Constructor which accepts our file selection manager.
	 */
	public FileInfoTableModel(FileSelectionManager manager)
	{
		mManager = manager;

		//
		// Allocate an initial full file list and make the filtered
		// file list be the same array to begin with.
		//
		mFiles = new ArrayAndMap(1000, manager.isCaseSensitiveFileNames());
		mFilteredFileList = mFiles.getArrayList();
	}


	/**
	 * Add a FileInfo row to the table.
	 * Returns true if the file was newly added.
	 * If the file already was in the table, nothing is done.
	 */
	public boolean addFileInfo(FileInfo fileInfo)
	{
		//
		// If there is no fileInfo or the name already is already in
		// the table then return false.
		//
		if ((fileInfo == null) || mFiles.exists(fileInfo.getKey()))
			return false;

		//
		// If there is no select list filter then add the file to the
		// full file list (which also adds to the filtered file list
		// since they are the same table in that case).
		//
		if (mSelectListFilter == null)
		{
			mFiles.put(fileInfo);

			int insertRow = fileInfo.getIndex();

			updateModifiedChangeCounter();

			return true;
		}

		//
		// There is a select list filter so that the full file list
		// and the filtered file lists are different objects.
		// Add the file to the full file list.
		//
		mFiles.put(fileInfo);

		//
		// Add this file if the file being in the select list matches
		// the shows files flag.
		//
		if (mSelectListFilter.isFileInList(fileInfo) ==
			mSelectListFilterShowsFiles)
		{
			mFilteredFileList.add(fileInfo);
		}

		updateModifiedChangeCounter();

		return true;
	}


	/**
	 * Return an iterator for the files in the table.
	 */
	public FileInfoIterator getFileInfoIterator()
	{
		return new FileInfoIterator(mFiles.getArrayList());
	}


	/**
	 * Return the full count of FileInfo objects we are managing.
	 * This is not the same as the number of rows due to filtering.
	 */
	public int getFileCount()
	{
		return mFiles.getCount();
	}


	/**
	 * Add a FileInfoColumn to the table if it does not already
	 * exist, while setting its preferred width to the default width
	 * for the FileInfoColumn.  Returns true if the column was
	 * actually added.
	 */
	public boolean addColumn(FileInfoColumn fileInfoColumn)
	{
		return addColumn(fileInfoColumn, 0);
	}


	/**
	 * Add a FileInfoColumn to the table if it does not already
	 * exist while saving its preferred width as specified.
	 * A negative or zero width uses the default width for the
	 * FileInfoColumn.  Returns true if the column was actually added.
	 */
	public boolean addColumn(FileInfoColumn fileInfoColumn, int width)
	{
		//
		// If the column is null or is already in the table then
		// return false.
		//
		if (fileInfoColumn == null)
			return false;

		if (mColumnList.contains(fileInfoColumn))
			return false;

		//
		// Add the column to the table and also add the width.
		//
		mColumnList.add(fileInfoColumn);

		if (width <= 0)
			width = fileInfoColumn.getColumnWidth();

		mColumnWidthList.add(new Integer(width));

		//
		// If this is a select list then add the column use to it.
		//
		if (fileInfoColumn instanceof SelectList)
		{
			SelectList selectList = (SelectList) fileInfoColumn;

			selectList.addSpecialUse(COLUMN_USE);
		}

		fireTableStructureChanged();

		return true;
	}


	/**
	 * Return whether a FileInfoColumn is used the table.
	 */
	public boolean isColumnUsed(FileInfoColumn fileInfoColumn)
	{
		return mColumnList.contains(fileInfoColumn);
	}


	/**
	 * Remove the indicated FileInfoColumn from the table.
	 * Nothing is done if the column was not being used.
	 * Returns true if the column was actually removed.
	 */
	public boolean removeColumn(FileInfoColumn fileInfoColumn)
	{
		//
		// See if the column is actually in use.
		//
		int index = mColumnList.indexOf(fileInfoColumn);

		if (index < 0)
			return false;

		//
		// It was used, so remove it and its width entry.
		//
		mColumnList.remove(index);
		mColumnWidthList.remove(index);

		//
		// If this column was a select list then remove its column use.
		//
		if (fileInfoColumn instanceof SelectList)
		{
			SelectList selectList = (SelectList) fileInfoColumn;

			selectList.removeSpecialUse(COLUMN_USE);
		}

		fireTableStructureChanged();

		return true;
	}


	/**
	 * Set the select list that the list of files are being filtered on
	 * along with the flag which indicates whether the files to be shown
	 * are in the select list or not in it.  A null value disables
	 * filtering and allows the full file list set for us to be displayed.
	 */
	public void setSelectListFilter(SelectList selectList,
		boolean showsFiles)
	{
		if (mSelectListFilter != null)
			mSelectListFilter.removeSpecialUse(FILTER_USE);

		mSelectListFilter = selectList;

		if (mSelectListFilter != null)
			mSelectListFilter.addSpecialUse(FILTER_USE);

		mSelectListFilterShowsFiles = showsFiles;

		filterFileList();
	}


	/**
	 * Return the select list filter that is being used.
	 * Returns null if there is no filter set.
	 */
	public SelectList getSelectListFilter()
	{
		return mSelectListFilter;
	}


	/**
	 * Return whether the select list filter is being used to show files.
	 */
	public boolean isSelectListFilterShowingFiles()
	{
		return mSelectListFilterShowsFiles;
	}


	/**
	 * Recalculate the filtered file list based on a change.
	 */
	public void filterFileList()
	{
		//
		// If there is no select list then make the filtered file
		// list the same array as the full file list.
		//
		if (mSelectListFilter == null)
		{
			mFilteredFileList = mFiles.getArrayList();

			updateDisplayedChangeCounter();

			return;
		}

		//
		// There is a select list filter.
		// Scan all of the files in the full list and check them against
		// the select list, and put them into the filtered file list if
		// they are selected.
		//
		int count = mFiles.getCount();

		mFilteredFileList = new ArrayList(count);

		for (int index = 0; index < count; index++)
		{
			FileInfo fileInfo = (FileInfo) mFiles.get(index);

			if (mSelectListFilter.isFileInList(fileInfo) ==
				mSelectListFilterShowsFiles)
			{
				mFilteredFileList.add(fileInfo);
			}
		}

		updateModifiedChangeCounter();
	}


	/**
	 * Overridden method to return the number of rows in the table.
	 * This is the filtered row count, and is not the total number of
	 * FileInfo objects that we are managing.
	 */
	public int getRowCount()
	{
		return mFilteredFileList.size();
	}


	/**
	 * Overridden method to return the number of columns in the table.
	 */
	public int getColumnCount()
	{
		return mColumnList.size();
	}


	/**
	 * Set the preferred column width for one of our columns.
	 * A non-positive value sets the width to the default for
	 * this FileInfoColumn.
	 */
	public void setPreferredColumnWidth(int col, int width)
	{
		if (width <= 0)
		{
			FileInfoColumn fileInfoColumn =
				(FileInfoColumn) mColumnList.get(col);

			width = fileInfoColumn.getColumnWidth();
		}

		mColumnWidthList.set(col, new Integer(width));
	}


	/**
	 * Get the preferred column width for one of our columns.
	 */
	public int getPreferredColumnWidth(int col)
	{
		Integer width = (Integer) mColumnWidthList.get(col);

		return width.intValue();
	}


	/**
	 * Overridden method to return the value at the indicated row
	 * and column of the table.
	 */
	public Object getValueAt(int row, int col)
	{
		//
		// Check the column and row numbers for validity.
		//
		if (((col < 0) || (col >= mColumnList.size())) ||
			((row < 0) || (row >= mFilteredFileList.size())))
		{
			return "???";
		}

		//
		// Get the file information for this row.
		//
		FileInfo fileInfo = (FileInfo) mFilteredFileList.get(row);

		//
		// Get the FileInfoColumn for this column.
		//
		FileInfoColumn fileInfoColumn =
			(FileInfoColumn) mColumnList.get(col);

		//
		// Return the column object to return the value for this
		// row object.
		//
		return fileInfoColumn.getValue(fileInfo);
	}


    /**
     * Overridden method to set the value at the indicated row and
     * column of the table.  We only allow this for the Boolean values
     * used in a Manual Select List.
     */
	public void setValueAt(Object value, int rowIndex, int columnIndex)
	{
		//
		// Check the column and row numbers and value for validity.
		//
		if (((columnIndex < 0) || (columnIndex >= mColumnList.size())) ||
			((rowIndex < 0) || (rowIndex >= mFilteredFileList.size())) ||
			!(value instanceof Boolean))
		{
			return;
		}

		//
		// Get the FileInfoColumn for this column.
		//
		FileInfoColumn fileInfoColumn =
			(FileInfoColumn) mColumnList.get(columnIndex);

		//
		// Make sure this column is a manual select list.
		//
		if (!(fileInfoColumn instanceof ManualSelectList))
			return;

		ManualSelectList selectList = (ManualSelectList) fileInfoColumn;

		//
		// Get the file information for this row.
		//
		FileInfo fileInfo = (FileInfo) mFilteredFileList.get(rowIndex);

		int fileIndex = fileInfo.getIndex();

		//
		// Set or clear the membership of this file in the select list.
		//
		if (((Boolean) value).booleanValue())
			selectList.addFileToList(fileIndex);
		else
			selectList.removeFileFromList(fileIndex);
	}


	/**
	 * Overridden method to return the name of a column.
	 */
    public String getColumnName(int col)
    {
    	FileInfoColumn fileInfoColumn =
    		(FileInfoColumn) mColumnList.get(col);

    	return fileInfoColumn.getHeader();
    }


	/**
	 * Overridden method to return the class of the column.
	 */
    public Class getColumnClass(int col)
    {
    	FileInfoColumn fileInfoColumn =
    		(FileInfoColumn) mColumnList.get(col);

    	return fileInfoColumn.getValueClass();
    }


	/**
	 * Overridden method to return whether a cell is editable.
	 */
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		FileInfoColumn fileInfoColumn =
			(FileInfoColumn) mColumnList.get(columnIndex);

		return !fileInfoColumn.isReadOnly();
	}


	/**
	 * Return whether a FileInfo exists for the specified file name.
	 */
	public boolean exists(String fileName)
	{
		return mFiles.exists(fileName);
	}


	/**
	 * Return the FileInfo object having the specified file name.
	 * This will find the object even if it is being filtered.
	 * Returns null if the file info entry is not defined.
	 */
	public FileInfo getFileInfo(String fileName)
	{
		return (FileInfo) mFiles.get(fileName);
	}


	/**
	 * Return the FileInfo object having the indicated index.
	 * This is the index into the full file table, ignoring any filtering
	 * that may be set.  This is not the same as the rows being viewed.
	 */
	public FileInfo getFileInfo(int index)
	{
		return (FileInfo) mFiles.get(index);
	}


	/**
	 * Return the FileInfo object having the indicated index into the
	 * viewed table.  This corresponds to the rows seen in the table.
	 * This is not the same as the the index into the full file table
	 * since filtering might be set.
	 */
	public FileInfo getFilteredFileInfo(int index)
	{
		return (FileInfo) mFilteredFileList.get(index);
	}


	/**
	 * Return an array of the current FileInfoColumn objects in use.
	 */
	public FileInfoColumn[] getFileInfoColumnArray()
	{
		int count = mColumnList.size();

		FileInfoColumn[] columnArray = new FileInfoColumn[count];

		mColumnList.toArray(columnArray);

		return columnArray;
	}


	/**
	 * Return whether the object has a modified change counter value
	 * which is higher then the specified one.  If so, then this means
	 * that the object has been modified since the time indicated
	 * by the test change counter value.
	 */
	public boolean isChangedSince(long testChangeCounter)
	{
		return mModifiedChangeCounter > testChangeCounter;
	}


	/**
	 * Return the object's current last modified change counter value.
	 */
	public long getModifiedChangeCounter()
	{
		return mModifiedChangeCounter;
	}


	/**
	 * Return the object's current last displayed change counter value.
	 */
	public long getDisplayedChangeCounter()
	{
		return mDisplayedChangeCounter;
	}


	/**
	 * Update the last modified change counter for the object to be the
	 * newest modified object managed by the file selection manager.
	 */
	public void updateModifiedChangeCounter()
	{
		mModifiedChangeCounter = mManager.getNextChangeCounter();
	}


	/**
	 * Return whether or not data for this object needs to be displayed.
	 * This is true if the last displayed change counter doesn't match
	 * the last modified change counter.
	 */
	public boolean isDisplayNeeded()
	{
		return mDisplayedChangeCounter != mModifiedChangeCounter;
	}


	/**
	 * Update the last displayed change counter to be the same as the
	 * last modified change counter value, and return whether or not
	 * there had been a change between the two values.
	 */
	public boolean updateDisplayedChangeCounter()
	{
		if (mDisplayedChangeCounter == mModifiedChangeCounter)
			return false;

		mDisplayedChangeCounter = mModifiedChangeCounter;

		return true;
	}


	/**
	 * Check all of the columns of the table to see if there are any
	 * columns are select lists which have changed since our last display.
	 * Returns true if so.
	 */
	public boolean isColumnUpdateNeeded()
	{
		for (int index = 0; index < mColumnList.size(); index++)
		{
			FileInfoColumn column = (FileInfoColumn) mColumnList.get(index);

			if (column instanceof SelectList)
			{
				SelectList selectList = (SelectList) column;

				if (selectList.isChangedSince(mDisplayedChangeCounter))
					return true;
			}
		}

		return false;
	}


	/**
	 * Check to see if the filter select list has changed since our last
	 * display.  Returns true if so wuth the new table model set.
	 */
	public boolean isFilterUpdateNeeded()
	{
		if (mSelectListFilter == null)
			return false;

		if (!mSelectListFilter.isChangedSince(mDisplayedChangeCounter))
			return false;

		filterFileList();

		return true;
	}


	/**
	 * Check for an update which requires the whole fileinfo table to
	 * be redrawn.  If so, then our last modified change counter value
	 * is updated to force us to display later.
	 */
	public void checkFullTableUpdateNeeded()
	{
		if (isFilterUpdateNeeded() || isColumnUpdateNeeded())
			updateModifiedChangeCounter();
	}


	/**
	 * Method called at appropriate times to update all of the rows of
	 * the file table which have been modified somehow.  We do this here
	 * since modifications of the file rows occur beneath this table model
	 * level.  The file info objects maintain a last modified counter and
	 * and last displayed counter.  We simply update those rows where the
	 * counters don't match while updating the last displayed counter to match.
	 */
	public void firePendingRowUpdates()
	{
		//
		// See if a full table update is needed.
		//
		checkFullTableUpdateNeeded();

		//
		// If the whole table is to be updated then just do that and
		// update the displayed change counter for each row since the
		// table update will make them all up to date.
		//
		if (updateDisplayedChangeCounter())
		{
			for (int index = 0; index < mFilteredFileList.size(); index++)
			{
				FileInfo fileInfo = (FileInfo) mFilteredFileList.get(index);

				fileInfo.updateDisplayedChangeCounter();
			}

			fireTableDataChanged();

			return;
		}

		//
		// The table as a whole does not need updating.
		// Loop over all of the displayed file info rows checking each one.
		//
		for (int index = 0; index < mFilteredFileList.size(); index++)
		{
			FileInfo fileInfo = (FileInfo) mFilteredFileList.get(index);

			//
			// If this row needs updating then do it.
			// This call will also update the displayed counter value.
			//
			if (fileInfo.updateDisplayedChangeCounter())
				fireTableRowsUpdated(index, index);
		}
	}
}
