/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection;

import java.util.BitSet;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.BufferedOutputStream;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import javax.swing.JOptionPane;
import javax.swing.JTable;

import java.awt.Component;
import java.awt.Color;

import fileselection.dialog.AddFileInfoColumnsDialog;
import fileselection.dialog.AddOrRemoveFromSelectListDialog;
import fileselection.dialog.FindFileNameDialog;
import fileselection.dialog.SearchDialog;
import fileselection.dialog.ChecksumDialog;
import fileselection.dialog.CommandDialog;
import fileselection.dialog.SetSelectListColorDialog;
import fileselection.dialog.ClearSelectListColorDialog;
import fileselection.dialog.SetFileInfoFilterDialog;
import fileselection.dialog.AddSelectListDialog;
import fileselection.dialog.RemoveFileInfoColumnsDialog;
import fileselection.dialog.SaveSelectListFileNamesDialog;
import fileselection.dialog.LoadSelectListFileNamesDialog;
import fileselection.dialog.ClearAllFilesFromSelectListDialog;
import fileselection.dialog.InteractiveFileDialog;
import fileselection.dialog.AddFilesDialog;
import fileselection.dialog.UpdateFileStatusDialog;
import fileselection.dialog.EditPathPrefixesDialog;
import fileselection.dialog.EditFileAssociationsDialog;
import fileselection.dialog.HelpDialog;

import fileselection.utilities.Utils;
import fileselection.utilities.Settings;


/**
 * Generic global action methods called when using menus or buttons.
 * These actions know that they are called in a GUI context and so can
 * bring up dialog boxes and progress monitors.
 */
public class MenuActions
{
	/**
	 * The default title for a warning dialog.
	 */
	public static final String DEFAULT_WARNING_TITLE =
		"Menu Action Failed";

	/**
	 * The file selection manager we call to perform actions.
	 */
	protected FileSelectionManager mManager;

	/**
	 * Our main controlling window.
	 */
	protected MainWindow mMainWindow;

	/**
	 * The parent component which we are a child of.
	 */
	protected Component mParent;


	/**
	 * Constructor which sets our parent main window.
	 */
	public MenuActions(MainWindow mainWindow)
	{
		mMainWindow = mainWindow;

		//
		// Extract values from the main window which we use.
		//
		mManager = mainWindow.getManager();
		mParent = mainWindow.getJFrame();
	}


	/**
	 * Action to exit from the application.
	 * This saves the current settings values before exiting.
	 */
	public void exit()
	{
		//
		// Try to update the settings values and save them to the
		// default settings file in the user's home area.
		//
		Settings settings = Settings.getInstance();

		settings.callSettingsSavers();
		settings.storeProperties();

		//
		// Now terminate the application.
		//
		System.exit(0);
	}


	/**
	 * Action to clear the colors for all select lists.
	 */
	public void clearAllSelectListColors()
	{
		//
		// Iterate over all of the select lists setting their
		// color table to null.
		//
		for (int index = 0; index < mManager.getSelectListCount();
			index++)
		{
			SelectList selectList = mManager.getSelectList(index);

			selectList.setColor(null);
		}

		//
		// Inform both the file table that it needs to be updated.
		//
		mManager.getFileInfoTableModel().updateModifiedChangeCounter();
	}


	/**
	 * Action to clear colors from a select list.
	 */
	public void clearSelectListColor()
	{
		//
		// Allocate the clear select list color dialog.
		//
		ClearSelectListColorDialog dialog =
			new ClearSelectListColorDialog(mMainWindow);

		//
		// Show the dialog and if it was cancelled then return.
		//
		if (!dialog.show())
			return;

		//
		// Get the select list array.
		//
		SelectList[] selectListArray = dialog.getSelectListArray();

		//
		// Clear the color for each select list.
		//
		for (int i = 0; i < selectListArray.length; i++)
			selectListArray[i].setColor(null);

		//
		// Inform the file table that it needs to be updated.
		//
		mManager.getFileInfoTableModel().updateModifiedChangeCounter();
	}


	/**
	 * Action to set the select list color.
	 */
	public void setSelectListColor()
	{
		//
		// Allocate the set select list color dialog.
		//
		SetSelectListColorDialog dialog =
			new SetSelectListColorDialog(mMainWindow);

		//
		// Show the dialog and if it was cancelled then return.
		//
		if (!dialog.show())
			return;

		//
		// Get the selected color and select list array.
		//
		Color color = dialog.getColor();
		SelectList[] selectListArray = dialog.getSelectListArray();

		//
		// Set the color for each select list.
		//
		for (int i = 0; i < selectListArray.length; i++)
			selectListArray[i].setColor(color);

		//
		// Inform the file info table that it needs to be updated.
		//
		mManager.getFileInfoTableModel().updateModifiedChangeCounter();
	}


	/**
	 * Action to add files to the file list.
	 */
	public void addFiles()
	{
		AddFilesDialog dialog = new AddFilesDialog(mMainWindow);
		dialog.runDialog();
	}


	/**
	 * Action to update the file status.
	 */
	public void updateFileStatus()
	{
		UpdateFileStatusDialog dialog =
			new UpdateFileStatusDialog(mMainWindow);

		dialog.runDialog();
	}


	/**
	 * Action to clear the changed flag in the file status.
	 */
	public void clearFileIsChanged()
	{
		mManager.clearFileIsChangedStatus();
	}


	/**
	 * Action to clear the new flag in the file status.
	 */
	public void clearFileIsNewFile()
	{
		mManager.clearFileIsNewStatus();
	}


	/**
	 * Action to set the file select list filter.
	 */
	public void setFileSelectListFilter()
	{
		SetFileInfoFilterDialog dialog =
			new SetFileInfoFilterDialog(mMainWindow);

		if (!dialog.show())
			return;

		SelectList selectList = dialog.getSelectList();

		boolean isSelectListShowingFiles =
			dialog.isSelectListShowingFiles();

		mManager.setFileSelectListFilter(selectList,
			isSelectListShowingFiles);
	}


	/**
	 * Action to clear the file select list filter.
	 */
	public void clearFileSelectListFilter()
	{
		mManager.setFileSelectListFilter(null, false);
	}


	/**
	 * Action to add a new temporary manual select list.
	 */
	public void addTempSelectList()
	{
		mManager.generateTemporaryManualSelectList();
	}


	/**
	 * Action to add a new manual select list.
	 */
	public void addManualSelectList()
	{
		//
		// Get the add select list dialog.
		//
		AddSelectListDialog dialog =
			new AddSelectListDialog(mMainWindow, null);

		//
		// Show the dialog and if it was cancelled then return.
		//
		if (!dialog.show("Add Manual Select List"))
			return;

		//
		// Get the name and description values that were entered.
		//
		String name = dialog.getSelectListName();
		String description = dialog.getSelectListDescription();

		//
		// Try to add the select list to the select list table.
		//
		if (mManager.addManualSelectList(name, description))
			return;

		//
		// The list could not be added, so inform the user.
		//
		showWarning("The select list name \"" + name +
			"\" is illegal or already exists");
	}


	/**
	 * Action to add a new calculation select list.
	 */
	public void addCalculationSelectList()
	{
		//
		// Get the add select list dialog.
		//
		AddSelectListDialog dialog =
			new AddSelectListDialog(mMainWindow, null);

		//
		// Show the dialog and if it was cancelled then return.
		//
		if (!dialog.show("Add Calculation Select List"))
			return;

		//
		// Get the name and description values that were entered.
		// The description is the expression to be evaluated.
		//
		String name = dialog.getSelectListName();
		String description = dialog.getSelectListDescription();

		//
		// Try to add the select list to the select list table.
		//
		if (mManager.addCalculationSelectList(name, description))
			return;

		//
		// The list could not be added, so inform the user.
		//
		showWarning("The select list name \"" + name +
			"\" is illegal, already exists, or has a bad expression");
	}


	/**
	 * Action to add columns to the FileInfo table.
	 */
	public void addFileColumns()
	{
		//
		// Get the add file columns dialog.
		//
		AddFileInfoColumnsDialog dialog =
			new AddFileInfoColumnsDialog(mMainWindow);

		//
		// Show the dialog and if it was cancelled then return.
		//
		if (!dialog.show())
			return;

		//
		// Get the list of columns to be added, if any.
		//
		FileInfoColumn[] columnObjects = dialog.getColumnObjects();

		if (columnObjects.length == 0)
			return;

		//
		// Store the current column widths so that they can be restored
		// after the change.
		//
		mMainWindow.storeFileInfoColumnWidths();

		//
		// Add the requested columns.
		//
		for (int index = 0; index < columnObjects.length; index++)
			mManager.addFileInfoColumn(columnObjects[index]);

		//
		// Restore the column widths.
		//
		mMainWindow.setFileInfoColumnWidths();
	}


	/**
	 * Action to remove columns from the FileInfo table.
	 */
	public void removeFileColumns()
	{
		//
		// Get the current array of columns.
		//
		FileInfoColumn[] currentColumns = mManager.getFileInfoColumnArray();

		//
		// Get the add file columns dialog.
		//
		RemoveFileInfoColumnsDialog dialog =
			new RemoveFileInfoColumnsDialog(mMainWindow);

		//
		// Show the dialog and if it was cancelled then return.
		//
		if (!dialog.show(currentColumns))
			return;

		//
		// Get the list of columns to be removed, if any.
		//
		FileInfoColumn[] removeColumns = dialog.getColumnObjects();
		
		if (removeColumns.length == 0)
			return;

		//
		// If all columns are to be removed then complain.
		//
		if (removeColumns.length >= currentColumns.length)
		{
			showWarning("Cannot remove all columns");

			return;
		}

		//
		// Store the current column widths so that they can be restored
		// after the change.
		//
		mMainWindow.storeFileInfoColumnWidths();

		//
		// Remove the requested columns.
		//
		for (int index = 0; index < removeColumns.length; index++)
			mManager.removeFileInfoColumn(removeColumns[index]);

		//
		// Restore the column widths.
		//
		mMainWindow.setFileInfoColumnWidths();
	}


	/**
	 * Edit the list of path prefixes.
	 */
	public void editPathPrefixes()
	{
		EditPathPrefixesDialog dialog = new EditPathPrefixesDialog(mMainWindow);

		dialog.show();
	}


	/**
	 * Edit the list of file associations.
	 */
	public void editFileAssociations()
	{
		EditFileAssociationsDialog dialog =
			new EditFileAssociationsDialog(mMainWindow);

		dialog.show();
	}


	/**
	 * Save the file names contained in one or more select lists.
	 */
	public void saveSelectListFileNames()
	{
		//
		// Run the dialog to get the parameters.
		//
		SaveSelectListFileNamesDialog dialog =
			new SaveSelectListFileNamesDialog(mMainWindow);

		if (!dialog.show())
			return;

		//
		// Extract the parameters from the dialog.
		//
		SelectList[] selectListArray = dialog.getSelectListArray();
		String saveFileName = dialog.getSaveFileName();
		boolean isSingleFileSave = dialog.isSingleFileSave();

		//
		// Make sure at least one select list was selected.
		//
		if ((selectListArray == null) || (selectListArray.length == 0))
		{
			showWarning("No select list selected");

			return;
		}

		//
		// Make sure there was a file name.
		//
		if ((saveFileName == null) || (saveFileName.length() == 0))
		{
			showWarning("No save file name specified");

			return;
		}

		//
		// Build a title string for the information dialog.
		//
		String title = ((selectListArray.length == 1) ?
			"Select List Saved" : "Select Lists Saved");

		//
		// Allocate a bitset so we can determine how many files
		// were written out.
		//
		BitSet bitSet = new BitSet(mManager.getFileCount());

		//
		// Find out about the save file.
		//
		File saveFile = new File(saveFileName);

		//
		// If we are writing the file list to one file then do that.
		//
		if (isSingleFileSave)
		{
			if (!writeFileList(saveFile, selectListArray, bitSet))
				return;

			//
			// Show an information message.
			//
			int fileCount = bitSet.cardinality();

			String fileText = ((fileCount == 1) ?
				"file name" : "file names");

			showInformation(title,
				"Total of " + fileCount + ' ' + fileText +
				" written to 1 list file");

			return;
		}

		//
		// We are writing to multiple files.
		// If the output path doesn't exist then create the
		// directory if the user agrees.
		//
		if (!saveFile.exists())
		{
			String question = "The directory " +
				saveFile.getAbsolutePath() +
				" does not exist.  Do you want to create it?";

			int result = JOptionPane.showConfirmDialog(mParent,
				question, "Create Directory", JOptionPane.YES_NO_OPTION);

			if (result != JOptionPane.YES_OPTION)
				return;

			if (!saveFile.mkdir())
			{
				showWarning("Failed to create directory \"" +
					saveFile.getAbsolutePath() + "\"");

				return;
			}
		}

		//
		// Make sure the output path is a directory.
		//
		if (!saveFile.isDirectory())
		{
			showWarning("Output path \"" + saveFile.getAbsolutePath() +
				"\" is not a directory");

			return;
		}

		//
		// Make an array to hold just one select list.
		//
		SelectList[] oneSelectList = new SelectList[1];

		//
		// Iterate over all of the select lists to write the files
		// belonging to them one by one.
		//
		for (int index = 0; index < selectListArray.length; index++)
		{
			//
			// Get the select list and make it the only entry in
			// the one-item array.
			//
			SelectList selectList = selectListArray[index];
			oneSelectList[0] = selectList;

			//
			// Create a file name by appending an extension to
			// the select list name.
			//
			String fileName = selectList.getName() + ".list";

			//
			// Get a file handle in the save directory having the
			// specified name.
			//
			File file = new File(saveFile, fileName);

			//
			// Try to write the file names in this select list to
			// the output file.
			//
			if (!writeFileList(file, oneSelectList, bitSet))
				return;
		}

		//
		// Put up a summary dialog to say what we did.
		//
		int fileCount = bitSet.cardinality();

		String fileText = ((fileCount == 1) ?
			"file name" : "file names");

		String listText = ((selectListArray.length == 1) ?
			"list file" : "list files");

		showInformation(title, "Total of " + fileCount + ' ' +
			fileText + " written to " + selectListArray.length +
			' ' + listText);
	}


	/**
	 * Write the files in the file list which belong to any of the
	 * select lists in the specified array.  The supplied bitset is
	 * marked to indicate those files which were written out.
	 * Returns true on success, or false with an error dialog shown.
	 */
	protected boolean writeFileList(File file,
		SelectList[] selectListArray, BitSet bitSet)
	{
		//
		// Get the end of line string for this system.
		//
		String lineSeparator = System.getProperty("line.separator");

		//
		// Now to open and write the file.
		// Catch exceptions so that we can clean up.
		//
		FileOutputStream outputStream = null;

		try
		{
			//
			// Open the file and get a writer to the file with a
			// buffered output stream for efficiency.
			//
			outputStream = new FileOutputStream(file);

			OutputStreamWriter writer = new OutputStreamWriter(
				new BufferedOutputStream(outputStream));

			//
			// Get an iterator to the file table.
			//
			FileInfoIterator iterator = mManager.getFileInfoIterator();

			//
			// Advance to the next file.
			//
			while (iterator.hasNext())
			{
				//
				// Get the next file.
				//
				FileInfo fileInfo = iterator.getNextFileInfo();

				//
				// If this file is in any of the selected select
				// lists then write out its name and a newline.
				//
				if (Utils.isFileInSelectLists(fileInfo,
					selectListArray))
				{
					writer.write(fileInfo.getFileName());
					writer.write(lineSeparator);

					bitSet.set(fileInfo.getIndex());
				}
			}

			//
			// Close the file.
			//
			writer.close();
			outputStream = null;
		}
		catch (Exception e)
		{
			//
			// Show the error message and return a count of -1.
			//
			showWarning("Error writing to file: " + e);

			return false;
		}
		finally
		{
			//
			// Make sure the output file is closed.
			//
			Utils.safeClose(outputStream);
		}

		return true;
	}


	/**
	 * Load the file names from a file into the selected manual
	 * select list.
	 */
	public void loadSelectListFileNamesItem()
	{
		//
		// Run the dialog to find the select list and file name.
		//
		LoadSelectListFileNamesDialog dialog =
			new LoadSelectListFileNamesDialog(mMainWindow);

		if (!dialog.show())
			return;

		//
		// Get the load file name and select list to be loaded.
		//
		String loadFileName = dialog.getLoadFileName();
		ManualSelectList selectList = dialog.getSelectList();

		//
		// Make sure there was a file name.
		//
		if ((loadFileName == null) || (loadFileName.length() == 0))
		{
			showWarning("No load file name specified");

			return;
		}

		//
		// Make sure a select list was specified.
		//
		if (selectList == null)
		{
			showWarning("No select list specified");

			return;
		}

		//
		// Catch exceptions while we read the file so that we
		// can clean up.
		//
		FileInputStream inputStream = null;

		try
		{
			//
			// Open the input file.
			//
			inputStream = new FileInputStream(loadFileName);

			//
			// Create a reader using buffered streams.
			//
			BufferedInputStream bufferedInputStream =
				new BufferedInputStream(inputStream);

			InputStreamReader inputStreamReader =
				new InputStreamReader(bufferedInputStream);

			BufferedReader reader =
				new BufferedReader(inputStreamReader);

			//
			// Read each line of the file.
			// Each line is one filename to be added to the file
			// table and assigned to the destination select list.
			//
			while (true)
			{
				//
				// Try to read the next line from the file which
				// is the next filename.
				//
				String fileName = reader.readLine();

				if (fileName == null)
					break;

				//
				// Add the file name to the file list if it was
				// not already present.
				//
				mManager.addFile(fileName);

				//
				// Get the file information for the added file.
				// and add the file to the select list.
				//
				FileInfo fileInfo = mManager.getFileInfo(fileName);

				selectList.addFileToList(fileInfo.getIndex());
			}

			//
			// Close the file.
			//
			inputStream.close();
			inputStream = null;
		}
		catch (Exception e)
		{
			//
			// Show the error message.
			//
			showWarning("Error reading file: " + e);

			return;
		}
		finally
		{
			//
			// Make sure the input file is closed.
			//
			Utils.safeClose(inputStream);
		}
	}


	/**
	 * Add the files which are currently selected to one
	 * or more select lists.
	 */
	public void addSelectedFilesToSelectList()
	{
		addOrRemoveSelectedFilesToSelectList(true);
	}


	/**
	 * Remove the files which are currently selected from one
	 * or more select lists.
	 */
	public void removeSelectedFilesFromSelectList()
	{
		addOrRemoveSelectedFilesToSelectList(false);
	}


	/**
	 * Add or remove the files which are currently selected to one
	 * or more select lists.
	 */
	protected void addOrRemoveSelectedFilesToSelectList(boolean isAdding)
	{
		//
		// Get the file info table and the number of selected rows.
		//
		JTable fileInfoTable = mMainWindow.getFileInfoJTable();

		int selectedRowCount = fileInfoTable.getSelectedRowCount();

		//
		// Make sure there is at least one file selected.
		//
		if (selectedRowCount == 0)
		{
			showWarning("There are no Files selected");

			return;
		}

		//
		// Run the dialog to ask the user what select lists to
		// be affected for the files.
		//
		AddOrRemoveFromSelectListDialog dialog =
			new AddOrRemoveFromSelectListDialog(mMainWindow);

		if (!dialog.show(selectedRowCount, isAdding))
			return;

		//
		// Get the array of the select lists to be affected.
		//
		ManualSelectList[] selectListArray =
			dialog.getManualSelectListArray();

		//
		// Get the array of the selected file list row numbers.
		//
		int[] fileInfoRows = fileInfoTable.getSelectedRows();

		//
		// Get the sorted table model for the table.
		// We need this to translate the selected rows in the table
		// model into the right FileInfo values.  (The file selection
		// manager doesn't know about that.)
		//
		FileInfoTableSorter sorter =
			(FileInfoTableSorter) fileInfoTable.getModel();

		//
		// Loop over all combinations of files and select lists setting
		// or clearing the file's bit in the select lists.
		//
		for (int i = 0; i < fileInfoRows.length; i++)
		{
			//
			// Get the FileInfo object for the next selected row.
			//
			FileInfo fileInfo =
				sorter.getFilteredFileInfo(fileInfoRows[i]);

			//
			// Get the bit number for the file.
			//
			int fileIndex = fileInfo.getIndex();

			//
			// Iterate over all of the selected select lists either
			// adding or removing this file.
			//
			for (int j = 0; j < selectListArray.length; j++)
			{
				SelectList selectList = selectListArray[j];

				if (isAdding)
					selectList.addFileToList(fileIndex);
				else
					selectList.removeFileFromList(fileIndex);
			}
		}

		//
		// Get the table model for the select list and notify its
		// listeners that we have changed things.
		//
		SelectListTableModel selectListTableModel =
			mManager.getSelectListTableModel();

		selectListTableModel.fireTableDataChanged();
	}


	/**
	 * Action to run the find file name operation.
	 */
	public void findFileNameOperation()
	{
		//
		// Build and run the find file name dialog.
		//
		FindFileNameDialog dialog = new FindFileNameDialog(mMainWindow);

		dialog.runDialog();
	}


	/**
	 * Action to run the search files operation.
	 */
	public void searchOperation()
	{
		//
		// Build and run the search dialog.
		//
		SearchDialog dialog = new SearchDialog(mMainWindow);

		dialog.runDialog();
	}


	/**
	 * Action to run the checksum files operation.
	 */
	public void checksumOperation()
	{
		//
		// Build and run the checksum dialog.
		//
		ChecksumDialog dialog = new ChecksumDialog(mMainWindow);

		dialog.runDialog();
	}


	/**
	 * Action to run the execute command  on files operation.
	 */
	public void commandOperation()
	{
		//
		// Build and run the command dialog.
		//
		CommandDialog dialog = new CommandDialog(mMainWindow);

		dialog.runDialog();
	}

	
	/**
	 * Action to interactively operate on files operation.
	 */
	public void interactiveFileOperation()
	{
		//
		// Build and run the interactive dialog.
		//
		InteractiveFileDialog dialog =
			new InteractiveFileDialog(mMainWindow);

		dialog.show();
	}


	/**
	 * Clear all files from the selected manual Select Lists.
	 */
	public void clearFilesFromSelectList()
	{
		//
		// Build and show the clear select lists dialog.
		//
		ClearAllFilesFromSelectListDialog dialog =
			new ClearAllFilesFromSelectListDialog(mMainWindow);

		if (!dialog.show())
			return;

		//
		// Get the list of manual select lists to be cleared.
		//
		ManualSelectList[] selectListArray =
			dialog.getManualSelectListArray();

		//
		// Perform the clearing.
		//
		for (int index = 0; index < selectListArray.length; index++)
			selectListArray[index].removeAllFilesFromList();
	}


	/**
	 * Show help on the specified topic.
	 */
	public void showHelp(String topic)
	{
		HelpDialog dialog = new HelpDialog(mMainWindow);

		dialog.show(topic);
	}


	/**
	 * Show a warning dialog with the default title and the
	 * indicated message.
	 */
	public void showWarning(String message)
	{
		showWarning(DEFAULT_WARNING_TITLE, message);
	}


	/**
	 * Show a warning dialog with the indicated title and message.
	 */
	public void showWarning(String title, String message)
	{
		JOptionPane.showMessageDialog(mParent, message, title,
			JOptionPane.WARNING_MESSAGE);
	}


	/**
	 * Show an informational dialog with the indicated title and message.
	 */
	public void showInformation(String title, String message)
	{
		JOptionPane.showMessageDialog(mParent, message, title,
			JOptionPane.INFORMATION_MESSAGE);
	}
}
