/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.JCheckBox;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.GridLayout;

import fileselection.MainWindow;

import fileselection.utilities.Utils;


/**
 * Class to build and show an option dialog that lets the user add a
 * new file association entry or edit an existing one.
 */
public class AddFileAssociationDialog
{
	/**
	 * The extension field.
	 */
	protected JTextField mExtensionField;

	/**
	 * The edit command field.
	 */
	protected JTextField mEditCommandField;

	/**
	 * The isText check box.
	 */
	protected JCheckBox mIsTextCheckBox;

	/**
	 * The extension that was entered.
	 */
	protected String mExtension;

	/**
	 * The edit command that was entered.
	 */
	protected String mEditCommand;

	/**
	 * The isText value that was entered.
	 */
	protected boolean mIsText;

	/**
	 * Whether we are editing old values.
	 */
	protected boolean mIsEditing;

	/**
	 * Our parent component.
	 */
	protected Component mParent;


	/**
	 * Constructor which sets the main window and our parent component.
	 * If the parent component is null then the main window is used.
	 */
	public AddFileAssociationDialog(MainWindow mainWindow, Component parent)
	{
		//
		// Extract the values we need.
		//
		if (parent != null)
			mParent = parent;
		else
			mParent = mainWindow.getJFrame();
	}


	/**
	 * Set the values of a file association to be seen in the dialog.
	 * This makes the dialog into an edit dialog.
	 */
	public void setEditingValues(String extension, boolean isText,
		String editCommand)
	{
		mExtension = extension;
		mIsText = isText;
		mEditCommand = editCommand;
		mIsEditing = true;
	}


	/**
	 * Show the Add File Association dialog.
	 * Returns true if the user used the OK button.
	 * After this the extension, isText flag, and edit command can be obtained.
	 */
	public boolean show()
	{
		//
		// Allocate the panel holding the message.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		//
		// Add the two sub-panels side by side.
		//
		mainPanel.add(buildLabelPanel(), BorderLayout.WEST);
		mainPanel.add(buildDataPanel(), BorderLayout.CENTER);

		//
		// Select the appropriate title for the dialog.
		//
		String title = (mIsEditing ?
			"Edit File Association" : "Add File Association");

		//
		// Show the dialog and get which button was used.
		//
		int result = JOptionPane.showConfirmDialog(mParent, mainPanel,
			title, JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return false;

		//
		// Extract the values.
		//
		mExtension = Utils.trimSpaces(mExtensionField.getText());
		mEditCommand = Utils.trimSpaces(mEditCommandField.getText());
		mIsText = mIsTextCheckBox.isSelected();

		return true;
	}


	/**
	 * Build a panel containing the labels for the data fields.
	 */
	protected JPanel buildLabelPanel()
	{
		JPanel panel = new JPanel(new GridLayout(0, 1));

		panel.add(new JLabel("Extension"));
		panel.add(new JLabel("Edit Command"));
		panel.add(new JLabel());

		return panel;
	}


	/**
	 * Build a panel containing the data entry components.
	 * This is two text fields and a checkbox.
	 */
	protected JPanel buildDataPanel()
	{
		JPanel panel = new JPanel(new GridLayout(0, 1));

		mExtensionField = new JTextField(20);

		mExtensionField.setText(mExtension);

		if (mIsEditing)
			mExtensionField.setEditable(false);

		mEditCommandField = new JTextField(60);

		mEditCommandField.setText(mEditCommand);

		mIsTextCheckBox = new JCheckBox("Is Text");

		mIsTextCheckBox.setSelected(mIsText);

		panel.add(mExtensionField);
		panel.add(mEditCommandField);
		panel.add(mIsTextCheckBox);

		return panel;
	}


	/**
	 * Return the extension that was entered.
	 */
	public String getExtension()
	{
		return mExtension;
	}


	/**
	 * Return the edit command that was entered.
	 */
	public String getEditCommand()
	{
		return mEditCommand;
	}


	/**
	 * Return whether the extension is for text.
	 */
	public boolean isText()
	{
		return mIsText;
	}
}
