/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JCheckBox;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.awt.BorderLayout;
import java.awt.Component;

import fileselection.FileSelectionManager;
import fileselection.MainWindow;
import fileselection.ManualSelectList;
import fileselection.utilities.Utils;


/**
 * Class to build and show an option dialog that lets the user
 * add a specified file to one or more select lists.  In addition,
 * if only one file is selected a tick box is enabled which says
 * that a button is to be created to perform the adding quickly.
 */
public class AddFileToSelectListDialog implements ListSelectionListener
{
	/**
	 * The manual select lists which were selected.
	 */
	protected ManualSelectList[] mSelectListArray;

	/**
	 * The JList which shows the select list.
	 */
	protected JList mSelectListList;

	/**
	 * The create button for select list button.
	 */
	protected JCheckBox mCreateButtonCheckBox;

	/**
	 * The main window.
	 */
	protected MainWindow mMainWindow;

	/**
	 * Our file selection manager.
	 */
	protected FileSelectionManager mManager;

	/**
	 * Our parent component.
	 */
	protected Component mParent;

	/**
	 * The select list table.
	 */
	protected JTable mSelectListTable;


	/**
	 * Constructor which sets the main window and parent component.
	 * If the parent is null then the main window is used.
	 */
	public AddFileToSelectListDialog(MainWindow mainWindow, Component parent)
	{
		//
		// Extract the values we need.
		//
		mMainWindow = mainWindow;
		mManager = mainWindow.getManager();
		mSelectListTable = mainWindow.getSelectListJTable();

		if (parent != null)
			mParent = parent;
		else
			mParent = mainWindow.getJFrame();
	}


	/**
	 * Show the Add Files to Select List Dialog.
	 * Returns true if the user used the OK button.
	 * After this the select lists can be obtained.
	 */
	public boolean show()
	{
		//
		// Allocate the panel holding the message with a border layout.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		//
		// Build the manual select list list.
		//
		mSelectListList = Utils.buildManualSelectListList(mManager, true, true);

		//
		// Add ourself as a selection listener so that we can enable
		// and disable the checkbox.
		//
		mSelectListList.addListSelectionListener(this);

		//
		// Wrap the list in a scroll pane and add it to the panel.
		//
		mainPanel.add(new JScrollPane(mSelectListList), BorderLayout.CENTER);

		//
		// For convenience, select those select lists in the select
		// list list which were selected in the select list table.
		//
		Utils.setSelectedSelectLists(mSelectListTable, mSelectListList);

		//
		// Create a check box for creating a button for the select list.
		//
		mCreateButtonCheckBox = new JCheckBox(
			"Create button for adding to this select list");

		mCreateButtonCheckBox.setEnabled(false);

		mainPanel.add(mCreateButtonCheckBox, BorderLayout.SOUTH);

		//
		// Show the dialog.
		//
		int result = JOptionPane.showConfirmDialog(mParent, mainPanel,
			"Add File to Select Lists", JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return false;

		//
		// Extract the select list table.
		//
		mSelectListArray = Utils.getManualSelectListArray(mSelectListList);

		//
		// Replace any special select lists in the table.
		//
		for (int index = 0; index < mSelectListArray.length; index++)
		{
			ManualSelectList selectList = mSelectListArray[index];

			selectList = Utils.handleSpecialManualSelectList(selectList,
				mMainWindow, null, "New Select List");

			if (selectList == null)
				return false;

			mSelectListArray[index] = selectList;
		}

		return true;
	}


	/**
	 * Implementation of ListSelectionListener method called when the
	 * selection changes on our list.
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		//
		// Enable the create button check box if we have exactly one
		// select list selected.
		//
		int[] indices = mSelectListList.getSelectedIndices();

		mCreateButtonCheckBox.setEnabled(indices.length == 1);
	}


	/**
	 * Return the array of ManualSelectList objects.
	 */
	public ManualSelectList[] getManualSelectListArray()
	{
		return mSelectListArray;
	}


	/**
	 * Return whether or not a button should be created.
	 */
	public boolean isCreateButtonWanted()
	{
		return mCreateButtonCheckBox.isSelected();
	}
}
