/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.util.BitSet;

import java.awt.BorderLayout;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JTable;

import fileselection.MainWindow;
import fileselection.ManualSelectList;

import fileselection.operation.CommandOperation;

import fileselection.utilities.Utils;


/**
 * Class to build and run a dialog to ask for a command string
 * to be applied to the files in the select lists, and output
 * select lists to be modified based on the exit status.
 */
public class CommandDialog extends OperationDialog
{
	/**
	 * The text field holding the command string.
	 */
	protected JTextField mCommandTextField;

	/**
	 * The JList of the select lists to run commands on.
	 */
	protected JList mCommandSelectListList;

	/**
	 * The components for the exit 0 status panel.
	 */
	protected ExitStatusComponents mExit0StatusComponents;

	/**
	 * The components for the exit 1 status panel.
	 */
	protected ExitStatusComponents mExit1StatusComponents;

	/**
	 * The components for the default exit status panel.
	 */
	protected ExitStatusComponents mDefaultExitStatusComponents;

	/**
	 * The actual command operation.
	 */
	protected CommandOperation mCommand;

	/**
	 * The select list table.
	 */
	protected JTable mSelectListTable;


	/**
	 * Constructor which sets the main window.
	 */
	public CommandDialog(MainWindow mainWindow)
	{
		super(mainWindow);

		//
		// Extract the values we need.
		//
		mSelectListTable = mainWindow.getSelectListJTable();
	}


	/**
	 * Run the command dialog by building an showing the dialog,
	 * and if the parameters were correct then running the command
	 * operation class, showing progress through a progress dialog.
	 */
	public void runDialog()
	{
		//
		// Show the dialog and if the user cancelled it then return.
		//
		if (!showDialog())
			return;

		//
		// Get the parameters from the dialog.
		//
		String command = getCommandText();
		BitSet fileBitSet = getCommandFileBitSet();

		//
		// Build up actions for the select lists.
		//
		CommandExitStatusAction exit0Action =
			buildExitStatusAction(mExit0StatusComponents);

		CommandExitStatusAction exit1Action =
			buildExitStatusAction(mExit1StatusComponents);

		CommandExitStatusAction defaultAction =
			buildExitStatusAction(mDefaultExitStatusComponents);

		//
		// Get the class which actually runs the commands and set
		// the parameters.
		//
		mCommand = new CommandOperation(mManager);

		mCommand.setCommandArguments(command);
		mCommand.setCommandFileBitSet(fileBitSet);
		mCommand.setDefaultReplacementString();
		mCommand.setExitStatusAction(exit0Action);
		mCommand.setExitStatusAction(exit1Action);
		mCommand.setDefaultExitStatusAction(defaultAction);

		//
		// Check the parameters and if they are wrong then display
		// an error message.
		//
		if (!mCommand.isReadyForExecution())
		{
			showWarning(mCommand.getErrorMessage());

			return;
		}

		//
		// Everything is ready.
		// Create the progress monitor dialog and inform the
		// command running class about it.
		//
		ProgressMonitorDialog progress =
			new ProgressMonitorDialog("Command Progress", this);

		mCommand.setListener(progress);

		//
		// Do the actual commands.
		// This operation does not block since it is executed in a
		// background thread.
		//
		if (!mCommand.startOperation())
			showWarning("Failed to execute operation");
	}


	/**
	 * Show the Command Dialog displaying a list of select lists
	 * that may be chosen from.  Returns true if the user used the OK
	 * button.  After this the selected SelectList can be obtained.
	 */
	public boolean showDialog()
	{
		//
		// Allocate the panel holding the message with a border layout.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		mainPanel.add(buildSourcePanel(), BorderLayout.NORTH);
		mainPanel.add(buildDestinationPanel(), BorderLayout.SOUTH);

		//
		// For convenience, select those select lists in the command
		// list which were selected in the select list table.
		//
		Utils.setSelectedSelectLists(mSelectListTable,
			mCommandSelectListList);

		//
		// Show the dialog and get the result.
		//
		int result = JOptionPane.showConfirmDialog(mParent, mainPanel,
			"Run Command for Files", JOptionPane.OK_CANCEL_OPTION);

		//
		// Return whether or not the OK button was used.
		//
		return (result == JOptionPane.OK_OPTION);
	}


	/**
	 * Called by the progress listener when the operation has
	 * completed to show the results in the Swing thread.
	 */
	public void showResults()
	{
		//
		// Update the tables before showing the results.
		//
		mManager.firePendingRowUpdates();

		//
		// Say how many files of each status we processed.
		//
		int totalCount = mCommand.getTotalFileCount();
		int processedCount = mCommand.getProcessedFileCount();
		int failedCount = mCommand.getFailedFileCount();
		int successCount = processedCount - failedCount;
		int unprocessedCount = totalCount - processedCount;

		String message =
			"Successful commands: " + successCount +
			", Failed commands: " + failedCount;

		if (unprocessedCount > 0)
			message += ", Unprocessed commands: " + unprocessedCount;

		JOptionPane.showMessageDialog(mParent, message,
			"Commands Completed", JOptionPane.INFORMATION_MESSAGE);
	}


	/**
	 * Return the command text string.
	 */
	public String getCommandText()
	{
		return mCommandTextField.getText();
	}


	/**
	 * Return the bit set specifying the files to run the
	 * command on, or null.
	 */
	public BitSet getCommandFileBitSet()
	{
		return Utils.getSelectedFileBitSet(mCommandSelectListList);
	}


	/**
	 * Build the source panel for the command dialog.
	 * This has the command select list, the command to be executed,
	 * and the options tick boxes.
	 */
	protected JPanel buildSourcePanel()
	{
		JPanel panel = new JPanel();

		panel.setBorder(BorderFactory.createTitledBorder("Command Parameters"));

		panel.add(buildSourceTextPanel());
		panel.add(buildSourceSelectListPanel());

		return panel;
	}


	/**
	 * Build the source select list panel.
	 * This contains a label at the top with a scrollable select list
	 * below that.
	 */
	protected JPanel buildSourceSelectListPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		//
		// Build the full select list list.
		//
		mCommandSelectListList =
			Utils.buildSelectListList(mManager, true);

		//
		// Wrap the list in a scroll pane.
		//
		JScrollPane scrollPane = new JScrollPane(mCommandSelectListList);

		//
		// Add a label to the top of the panel with the select
		// list underneath that.
		//
		panel.add(new JLabel("Command Select List"), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Build the source text panel.
	 * This has a label, a text field, and option buttons.
	 */
	protected JPanel buildSourceTextPanel()
	{
		JPanel panel = new JPanel();

		BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);

		panel.setLayout(layout);

		JLabel label = new JLabel("Command to Execute");

		mCommandTextField = new JTextField(40);

		panel.add(Box.createVerticalGlue());
		panel.add(label);
		panel.add(mCommandTextField);
		panel.add(Box.createVerticalGlue());

		return panel;
	}


	/**
	 * Build the destination panel.
	 * This contains the destination select list sub-panels for
	 * the exit 0 status, exit 1 status, and default exit status.
	 */
	protected JPanel buildDestinationPanel()
	{
		//
		// Allocate the components for the sub-panels.
		//
		mExit0StatusComponents = buildExitStatusComponents(
			"Exit Status 0 List", 0, false);

		mExit1StatusComponents = buildExitStatusComponents(
			"Exit Status 1 List", 1, true);

		mDefaultExitStatusComponents = buildExitStatusComponents(
			"Other Exit Status List", -1, true);

		//
		// Allocate the destination panel and fill it with the
		// sub-panels.
		//
		JPanel panel = new JPanel();

		panel.setBorder(BorderFactory.createTitledBorder(
			"Result Parameters"));

		panel.add(mExit0StatusComponents.mPanel);
		panel.add(mExit1StatusComponents.mPanel);
		panel.add(mDefaultExitStatusComponents.mPanel);

		return panel;
	}


	/**
	 * Build an exit status action object from the current state
	 * of the specified exit status components.
	 */
	protected CommandExitStatusAction buildExitStatusAction(
		ExitStatusComponents components)
	{
		//
		// Get the select list to be used for this exit status.
		// This handles the special <NEW> and <TEMP> select lists.
		//
		ManualSelectList selectList = Utils.getSelectedManualSelectList(
			mMainWindow, components.mSelectListList,
				"New " + components.mTitle);

		boolean isClearSelectList =
			components.mClearSelectListCheckBox.isSelected();

		boolean isFailure = components.mIsFailureCheckBox.isSelected();

		//
		// Allocate and return the action from these values.
		//
		CommandExitStatusAction action =
			new CommandExitStatusAction(components.mExitStatus,
				isFailure, selectList, isClearSelectList);

		return action;
	}


	/**
	 * Build the set of components for one exit status panel.
	 * This consists of a label, a select list lister, and a
	 * couple of checkboxes.
	 */
	protected ExitStatusComponents buildExitStatusComponents(
		String title, int exitStatus, boolean isFailure)
	{
		//
		// Allocate the panel to hold the components.
		//
		JPanel panel = new JPanel(new BorderLayout());

		//
		// Build the manual select list list and wrap it.
		//
		JList selectListList =
			Utils.buildManualSelectListList(mManager, false, true);

		JScrollPane scrollPane = new JScrollPane(selectListList);

		//
		// Allocate the check boxes.
		//
		JCheckBox clearCheckBox = new JCheckBox("Clear Select List");
		JCheckBox failureCheckBox = new JCheckBox("Status indicates failure");

		failureCheckBox.setSelected(isFailure);

		//
		// Put the check boxes into a vertical box.
		//
		JPanel checkBoxPanel = new JPanel();

		BoxLayout layout = new BoxLayout(checkBoxPanel, BoxLayout.Y_AXIS);

		checkBoxPanel.setLayout(layout);

		checkBoxPanel.add(clearCheckBox);
		checkBoxPanel.add(failureCheckBox);

		//
		// Add a label at the top, the select list in the center,
		// and the check boxes panel at the bottom.
		//
		panel.add(new JLabel(title), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(checkBoxPanel, BorderLayout.SOUTH);

		//
		// Build and return the components object.
		//
		ExitStatusComponents components = new ExitStatusComponents();

		components.mExitStatus = exitStatus;
		components.mTitle = title;
		components.mPanel = panel;
		components.mSelectListList = selectListList;
		components.mClearSelectListCheckBox = clearCheckBox;
		components.mIsFailureCheckBox = failureCheckBox;

		return components;
	}


	/**
	 * Class to hold components used for one exit status panel.
	 * This contains the container panel, the select list JList,
	 * the clear select list checkbox, and the failure checkbox.
	 */
	class ExitStatusComponents
	{
		public int mExitStatus;
		public String mTitle;
		public JPanel mPanel;
		public JList mSelectListList;
		public JCheckBox mClearSelectListCheckBox;
		public JCheckBox mIsFailureCheckBox;
	}
}
