/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import java.util.BitSet;

import java.awt.BorderLayout;
import java.awt.GridLayout;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;

import fileselection.FileSelectionManager;
import fileselection.MainWindow;
import fileselection.ManualSelectList;

import fileselection.operation.FindFileNameOperation;

import fileselection.utilities.Utils;


/**
 * Class to build and run a dialog to look for file name patterns.
 * This asks for the search string, affected select lists, and other
 * information for the search.  Since pattern searching is so fast this
 * doesn't actually run in a background thread like other operations do.
 */
public class FindFileNameDialog extends OperationDialog
{
	/**
	 * The text field holding the search string.
	 */
	protected JTextField mSearchTextField;

	/**
	 * The check box for doing regular expressions.
	 */
	protected JCheckBox mRegularExpressionCheckBox;

	/**
	 * The radio button to match the last file name component.
	 */
	protected JRadioButton mLastComponentRadioButton;

	/**
	 * The radio button to match the base file name.
	 */
	protected JRadioButton mBaseNameRadioButton;

	/**
	 * The radio button to match the extension.
	 */
	protected JRadioButton mExtensionRadioButton;

	/**
	 * The radio button to match the full path name.
	 */
	protected JRadioButton mFullPathRadioButton;

	/**
	 * The radio button to match the directory path name.
	 */
	protected JRadioButton mDirectoryPathRadioButton;

	/**
	 * The radio button to match any individual path component.
	 */
	protected JRadioButton mAnyComponentRadioButton;

	/**
	 * The button group for the radio buttons.
	 */
	protected ButtonGroup mButtonGroup;

	/**
	 * The check box for doing case sensitive search.
	 */
	protected JCheckBox mCaseSensitiveCheckBox;

	/**
	 * The check box for complete match required.
	 */
	protected JCheckBox mCompleteMatchCheckBox;

	/**
	 * The check box for clearing matching files.
	 */
	protected JCheckBox mClearMatchingCheckBox;

	/**
	 * The check box for clearing not matching files.
	 */
	protected JCheckBox mClearNotMatchingCheckBox;

	/**
	 * The JList of the search select lists.
	 */
	protected JList mFindFileSelectListList;

	/**
	 * The JList of the matching files select lists.
	 */
	protected JList mMatchingSelectListList;

	/**
	 * The The JList of the not matching files select lists.
	 */
	protected JList mNotMatchingSelectListList;

	/**
	 * The select list for matching files, or null.
	 */
	protected ManualSelectList mMatchingSelectList;

	/**
	 * The select list for not matching files, or null.
	 */
	protected ManualSelectList mNotMatchingSelectList;

	/**
	 * The file selection manager.
	 */
	protected FileSelectionManager mManager;

	/**
	 * The select list table.
	 */
	protected JTable mSelectListTable;

	/**
	 * The actual search operation.
	 */
	protected FindFileNameOperation mFindFile;


	/**
	 * Constructor which sets the main window.
	 */
	public FindFileNameDialog(MainWindow mainWindow)
	{
		super(mainWindow);

		//
		// Extract other values we need.
		//
		mManager = mainWindow.getManager();
		mSelectListTable = mainWindow.getSelectListJTable();
	}


	/**
	 * Run the find file name dialog by building an showing the dialog,
	 * and if the parameters were correct then running the find file
	 * operation class directly in our own thread.
	 */
	public void runDialog()
	{
		//
		// Show the dialog and if the user cancelled it then return.
		//
		if (!showDialog())
			return;

		//
		// Get the parameters from the dialog.
		//
		String searchText = getSearchText();

		BitSet searchFileBitSet = getSearchFileBitSet();

		boolean isCaseSensitive = isCaseSensitive();
		boolean isRegularExpression = isRegularExpressionSearch();
		boolean isCompleteMatch = isCompleteMatch();
		boolean isClearMatching = isMatchingSelectListCleared();
		boolean isClearNotMatching = isNotMatchingSelectListCleared();
		int searchType = getSearchType();

		//
		// Allocate the find file name class and set the parameters.
		//
		mFindFile = new FindFileNameOperation(mManager);

		mFindFile.setSearchType(searchType);
		mFindFile.setSearchPattern(searchText);
		mFindFile.setSearchFileBitSet(searchFileBitSet);
		mFindFile.setIsCaseSensitive(isCaseSensitive);
		mFindFile.setIsRegularExpression(isRegularExpression);
		mFindFile.setIsCompleteMatch(isCompleteMatch);
		mFindFile.setIsMatchingSelectListCleared(isClearMatching);
		mFindFile.setIsNotMatchingSelectListCleared(isClearNotMatching);
		mFindFile.setMatchingSelectList(mMatchingSelectList);
		mFindFile.setNotMatchingSelectList(mNotMatchingSelectList);

		//
		// Check the parameters and if they are wrong then display
		// an error message.
		//
		if (!mFindFile.isReadyForExecution())
		{
			showWarning(mFindFile.getErrorMessage());

			return;
		}

		//
		// Everything is ready.  Do the actual find file operation.
		// Since this is so fast we don't have to run it in a background
		// thread.
		//
		mFindFile.runOperation();

		//
		// Show the results.
		//
		showResults();
	}


	/**
	 * Show the Set File Info Filter Dialog displaying a list of
	 * select lists that may be chosen from.  Returns true if the
	 * user used the OK button.  After this the selected SelectList
	 * can be obtained.
	 */
	protected boolean showDialog()
	{
		//
		// Allocate the panel holding the message with a border layout.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		mainPanel.add(buildSourcePanel(), BorderLayout.NORTH);
		mainPanel.add(buildDestinationPanel(), BorderLayout.SOUTH);

		//
		// For convenience, select those select lists in the search
		// list which were selected in the select list table.
		//
		Utils.setSelectedSelectLists(mSelectListTable,
			mFindFileSelectListList);

		//
		// Show the dialog and get which button was used.
		//
		int result = JOptionPane.showConfirmDialog(mParent, mainPanel,
			"Find File Names", JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return false;

		//
		// Collect information from the dialog and replace special
		// select lists with normal ones.
		//
		mMatchingSelectList = Utils.getSelectedManualSelectList(
			mMainWindow, mMatchingSelectListList,
			"New Select List for Matching Files");

		mNotMatchingSelectList = Utils.getSelectedManualSelectList(
			mMainWindow, mNotMatchingSelectListList,
			"New Select List for Not Matching Files");

		return true;
	}


	/**
	 * Called by the progress listener when the operation has
	 * completed to show the results in the Swing thread.
	 */
	public void showResults()
	{
		//
		// Update the tables before showing the results.
		//
		mManager.firePendingRowUpdates();

		//
		// Say how many files of each status we processed.
		//
		String message =
			"File Names Matched: " +	mFindFile.getMatchingFileCount() +
			", Not Matched: " + mFindFile.getNotMatchingFileCount();

		JOptionPane.showMessageDialog(mParent, message,
			"Find File Name Complete", JOptionPane.INFORMATION_MESSAGE);
	}


	/**
	 * Return the search text string.
	 */
	public String getSearchText()
	{
		return mSearchTextField.getText();
	}


	/**
	 * Return whether the search is to be case sensitive.
	 */
	public boolean isCaseSensitive()
	{
		return mCaseSensitiveCheckBox.isSelected();
	}


	/**
	 * Return whether the search is to use regular expressions.
	 */
	public boolean isRegularExpressionSearch()
	{
		return mRegularExpressionCheckBox.isSelected();
	}


	/**
	 * Return whether or not the complete value must be matched.
	 */
	public boolean isCompleteMatch()
	{
		return mCompleteMatchCheckBox.isSelected();
	}


	/**
	 * Return the bit set for the files being searched, or null.
	 */
	public BitSet getSearchFileBitSet()
	{
		return Utils.getSelectedFileBitSet(mFindFileSelectListList);
	}


	/**
	 * Return the select list for matching files (or null).
	 */
	public ManualSelectList getMatchingSelectList()
	{
		return mMatchingSelectList;
	}


	/**
	 * Return the select list for not matching files (or null).
	 */
	public ManualSelectList getNotMatchingSelectList()
	{
		return mNotMatchingSelectList;
	}


	/**
	 * Return whether the matching select list is to be cleared.
	 */ 
	public boolean isMatchingSelectListCleared()
	{
		return mClearMatchingCheckBox.isSelected();
	}


	/**
	 * Return whether the not matching select list is to be cleared.
	 */ 
	public boolean isNotMatchingSelectListCleared()
	{
		return mClearNotMatchingCheckBox.isSelected();
	}


	/**
	 * Build the source panel for the search.
	 * This has the search select list, the text to be searched,
	 * and the options tick boxes.
	 */
	protected JPanel buildSourcePanel()
	{
		JPanel panel = new JPanel();

		panel.setBorder(BorderFactory.createTitledBorder(
			"Find Parameters"));

		panel.add(buildSearchOptionsPanel());
		panel.add(buildSourceSelectListPanel());

		return panel;
	}


	/**
	 * Build the source select list panel.
	 * This contains a label at the top with a scrollable select list
	 * below that.
	 */
	protected JPanel buildSourceSelectListPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		//
		// Build the full select list list.
		//
		mFindFileSelectListList =
			Utils.buildSelectListList(mManager, true);

		//
		// Wrap the list in a scroll pane.
		//
		JScrollPane scrollPane = new JScrollPane(mFindFileSelectListList);

		//
		// Add a label to the top of the panel with the select
		// list underneath that.
		//
		panel.add(new JLabel("File Name List"), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);

		return panel;
	}


	/**
	 * Build the source text panel.
	 * This has a label, a text field, and option buttons.
	 */
	protected JPanel buildSearchOptionsPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		panel.add(buildSourceTextPanel(), BorderLayout.NORTH);
		panel.add(new JPanel(), BorderLayout.CENTER);
		panel.add(buildFileComponentRadioButtonPanel(), BorderLayout.SOUTH);

		return panel;
	}


	protected JPanel buildSourceTextPanel()
	{
		JPanel panel = new JPanel(new GridLayout(0, 1));

		JLabel label = new JLabel("Text or Regular Expression to Find");

		mSearchTextField = new JTextField(20);

		mRegularExpressionCheckBox =
			new JCheckBox("Regular expression match");

		mCaseSensitiveCheckBox =
			new JCheckBox("Case sensitive match");

		mCompleteMatchCheckBox =
			new JCheckBox("Match complete value");

		panel.add(label);
		panel.add(mSearchTextField);
		panel.add(mRegularExpressionCheckBox);
		panel.add(mCaseSensitiveCheckBox);
		panel.add(mCompleteMatchCheckBox);

		return panel;
	}


	/**
	 * Build the panel holding the set of radio buttons.
	 */
	protected JPanel buildFileComponentRadioButtonPanel()
	{
		//
		// Build a titled panel with a vertical grid.
		//
		JPanel panel = new JPanel(new GridLayout(0, 2));

		panel.setBorder(new TitledBorder("Value to be matched"));

		//
		// Allocate a button group.
		//
		mButtonGroup = new ButtonGroup();

		//
		// Allocate the radio buttons for the button group.
		//
		mLastComponentRadioButton = new JRadioButton("Last component");
		mBaseNameRadioButton = new JRadioButton("Base file name");
		mExtensionRadioButton = new JRadioButton("Extension");
		mFullPathRadioButton = new JRadioButton("Full path");
		mDirectoryPathRadioButton = new JRadioButton("Directory path");
		mAnyComponentRadioButton = new JRadioButton("Any component");

		//
		// Add the radio buttons to the button group.
		//
		mButtonGroup.add(mLastComponentRadioButton);
		mButtonGroup.add(mBaseNameRadioButton);
		mButtonGroup.add(mExtensionRadioButton);
		mButtonGroup.add(mFullPathRadioButton);
		mButtonGroup.add(mDirectoryPathRadioButton);
		mButtonGroup.add(mAnyComponentRadioButton);

		//
		// Set the last file name component as the default search type.
		//
		mLastComponentRadioButton.setSelected(true);

		//
		// Add the radio buttons to the panel and return it.
		//
		panel.add(mLastComponentRadioButton);
		panel.add(mAnyComponentRadioButton);
		panel.add(mBaseNameRadioButton);
		panel.add(mExtensionRadioButton);
		panel.add(mFullPathRadioButton);
		panel.add(mDirectoryPathRadioButton);

		return panel;
	}


	/**
	 * Buuild the destination panel.
	 * This contains the three destination select list panels for
	 * the matching and not matching.
	 */
	protected JPanel buildDestinationPanel()
	{
		JPanel panel = new JPanel();

		panel.setBorder(BorderFactory.createTitledBorder("Result Parameters"));

		panel.add(buildMatchingSelectListPanel());
		panel.add(buildNotMatchingSelectListPanel());

		return panel;
	}

	
	/**
	 * Build a destination panel for matching files.
	 * This has a title, a scrolled select list list, and a tick box.
	 */
	protected JPanel buildMatchingSelectListPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		//
		// Build the manual select list list and wrap it.
		//
		mMatchingSelectListList =
			Utils.buildManualSelectListList(mManager, false, true);

		JScrollPane scrollPane =
			new JScrollPane(mMatchingSelectListList);

		//
		// Allocate a check box.
		//
		mClearMatchingCheckBox = new JCheckBox("Clear Select List");

		//
		// Add a label at the top, the select list in the center,
		// and the check box at the bottom.
		//
		panel.add(new JLabel("Matching List"), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(mClearMatchingCheckBox, BorderLayout.SOUTH);

		return panel;
	}


	/**
	 * Build a destination panel for not matching files.
	 * This has a title, a scrolled select list list, and a tick box.
	 */
	protected JPanel buildNotMatchingSelectListPanel()
	{
		JPanel panel = new JPanel(new BorderLayout());

		//
		// Build the manual select list list and wrap it.
		//
		mNotMatchingSelectListList =
			Utils.buildManualSelectListList(mManager, false, true);

		JScrollPane scrollPane =
			new JScrollPane(mNotMatchingSelectListList);

		//
		// Allocate a check box.
		//
		mClearNotMatchingCheckBox = new JCheckBox("Clear Select List");

		//
		// Add a label at the top, the select list in the center,
		// and the check box at the bottom.
		//
		panel.add(new JLabel("Not Matching List"), BorderLayout.NORTH);
		panel.add(scrollPane, BorderLayout.CENTER);
		panel.add(mClearNotMatchingCheckBox, BorderLayout.SOUTH);

		return panel;
	}


	/**
	 * Return the search type to be used according to our radio buttons.
	 */
	protected int getSearchType()
	{
		if (mBaseNameRadioButton.isSelected())
			return FindFileNameOperation.SEARCH_TYPE_BASE_NAME;

		if (mExtensionRadioButton.isSelected())
			return FindFileNameOperation.SEARCH_TYPE_EXTENSION;

		if (mFullPathRadioButton.isSelected())
			return FindFileNameOperation.SEARCH_TYPE_FULL_PATH;

		if (mDirectoryPathRadioButton.isSelected())
			return FindFileNameOperation.SEARCH_TYPE_DIRECTORY_PATH;

		if (mAnyComponentRadioButton.isSelected())
			return FindFileNameOperation.SEARCH_TYPE_ANY_COMPONENT;

		//
		// Nothing else is selected, so assume the last component search.
		//
		return FindFileNameOperation.SEARCH_TYPE_LAST_COMPONENT;
	}
}
