/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.dialog;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JColorChooser;
import javax.swing.JOptionPane;
import javax.swing.JTable;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;

import fileselection.FileSelectionManager;
import fileselection.MainWindow;
import fileselection.SelectList;
import fileselection.utilities.Utils;


/**
 * Class to build and show an option dialog that lets the user
 * set the color which is associated with one or more select lists.
 */
public class SetSelectListColorDialog
{
	/**
	 * The select lists which were selected.
	 */
	protected SelectList[] mSelectListArray;

	/**
	 * The JList which shows the select list.
	 */
	protected JList mSelectListList;

	/**
	 * The JColor chooser.
	 */
	protected JColorChooser mColorChooser;

	/**
	 * The selected color.
	 */
	protected Color mColor;

	/**
	 * Our file selection manager.
	 */
	protected FileSelectionManager mManager;

	/**
	 * Our parent component.
	 */
	protected Component mParent;

	/**
	 * The select list table.
	 */
	protected JTable mSelectListTable;


	/**
	 * Constructor which sets the main window.
	 */
	public SetSelectListColorDialog(MainWindow mainWindow)
	{
		//
		// Extract the values we need.
		//
		mManager = mainWindow.getManager();
		mParent = mainWindow.getJFrame();
		mSelectListTable = mainWindow.getSelectListJTable();
	}


	/**
	 * Show the Set Select List Color Dialog.
	 * Returns true if the user used the OK button.
	 * After this the select lists and color can be obtained.
	 */
	public boolean show()
	{
		//
		// Allocate the panel holding the message with a border layout.
		//
		JPanel mainPanel = new JPanel(new BorderLayout());

		mainPanel.add(buildSelectListPanel(), BorderLayout.WEST);
		mainPanel.add(buildColorChooserPanel(), BorderLayout.EAST);

		//
		// For convenience, select those select lists in the select
		// list list which were selected in the select list table.
		//
		Utils.setSelectedSelectLists(mSelectListTable, mSelectListList);

		//
		// Show the dialog and get which button was used.
		//
		int result = JOptionPane.showConfirmDialog(mParent, mainPanel,
			"Set Select List Color", JOptionPane.OK_CANCEL_OPTION);

		//
		// If the dialog was cancelled then return false.
		//
		if (result != JOptionPane.OK_OPTION)
			return false;

		//
		// Extract the select list table and the color chosen.
		//
		Object[] selectObjects = mSelectListList.getSelectedValues();

		mSelectListArray = new SelectList[selectObjects.length];

		for (int i = 0; i < selectObjects.length; i++)
			mSelectListArray[i] = (SelectList) selectObjects[i];

		mColor = mColorChooser.getColor();

		return true;
	}


	/**
	 * Build a panel containing a color chooser.
	 */
	protected JPanel buildColorChooserPanel()
	{
		//
		// Allocate a panel.
		//
		JPanel panel = new JPanel();

		//
		// Allocate the color chooser.
		//
		mColorChooser = new JColorChooser(Color.WHITE);

		//
		// Add the color chooser to the panel.
		//
		panel.add(mColorChooser);

		//
		// Return the panel.
		//
		return panel;
	}


	/**
	 * Build a panel showing the available select lists.
	 */
	protected JPanel buildSelectListPanel()
	{
		//
		// Allocate a panel.
		//
		JPanel panel = new JPanel();

		//
		// Build the full select list list.
		//
		mSelectListList = Utils.buildSelectListList(mManager, true);

		//
		// Wrap the list in a scroll pane and add it to the panel.
		//
		panel.add(new JScrollPane(mSelectListList));

		//
		// Return the panel.
		//
		return panel;
	}


	/**
	 * Return the array of SelectList objects to be set.
	 */
	public SelectList[] getSelectListArray()
	{
		return mSelectListArray;
	}


	/**
	 * Return the selected color.
	 * Returns null if the color is to be cleared.
	 */
	public Color getColor()
	{
		return mColor;
	}
}
