/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.operation;

import java.io.File;
import java.io.FileInputStream;

import java.security.MessageDigest;

import java.util.BitSet;
import java.util.Arrays;

import fileselection.FileInfo;
import fileselection.FileSelectionManager;
import fileselection.ManualSelectList;
import fileselection.utilities.Utils;


/**
 * The checksum operation for the files in the file table.
 * This calculates and sets the checksum for the indicated files.
 * Directories can be checksummed also, which is defined as the result
 * of the checksumming of the sorted list of file names in the directory.
 * Select lists can be set to indicate which files succeeded or failed.
 */
public class ChecksumOperation extends AbstractOperation
{
	/**
	 * The bitset which indicates which files are to be checksummed.
	 */
	protected BitSet mFileBitSet;

	/**
	 * The manual select list which gets the files which were successful.
	 * This can be null if this result is not desired.
	 */
	protected ManualSelectList mSuccessSelectList;

	/**
	 * The manual select list which gets the files which fail.
	 * This can be null if this result is not desired.
	 */
	protected ManualSelectList mFailureSelectList;

	/**
	 * The new bit set for the success select list.
	 */
	protected BitSet mSuccessBitSet;

	/**
	 * The new bit set for the failure select list.
	 */
	protected BitSet mFailureBitSet;

	/**
	 * Whether we clear or append to the success select list.
	 */
	protected boolean mIsSuccessSelectListCleared;

	/**
	 * Whether we clear or append to the failure select list.
	 */
	protected boolean mIsFailureSelectListCleared;

	/**
	 * The message digest algorithm.
	 */
	protected MessageDigest mMessageDigest;


	/**
	 * Constructor which sets our file selection manager.
	 */
	public ChecksumOperation(FileSelectionManager manager)
	{
		super(manager);
	}


	/**
	 * Set the bit set which specifies the files to checksum.
	 */
	public void setChecksumFileBitSet(BitSet fileBitSet)
	{
		mFileBitSet = fileBitSet;
	}


	/**
	 * Set the manual select list which gets the files which succeeded.
	 * This can be null if this result is not desired.
	 */
	public void setSuccessSelectList(ManualSelectList selectList)
	{
		mSuccessSelectList = selectList;
	}


	/**
	 * Set the manual select list which gets the files which fail.
	 * This can be null if this result is not desired.
	 */
	public void setFailureSelectList(ManualSelectList selectList)
	{
		mFailureSelectList = selectList;
	}


	/**
	 * Whether we clear or append to the success select list.
	 */
	public void setIsSuccessSelectListCleared(boolean isCleared)
	{
		mIsSuccessSelectListCleared = isCleared;
	}


	/**
	 * Whether we clear or append to the failure select list.
	 */
	public void setIsFailureSelectListCleared(boolean isCleared)
	{
		mIsFailureSelectListCleared = isCleared;
	}


	/**
	 * Return the count of files which were successful.
	 */
	public int getSuccessFileCount()
	{
		return mSuccessBitSet.cardinality();
	}


	/**
	 * Return the count of files which we failed to checksum.
	 */
	public int getFailureFileCount()
	{
		return mFailureBitSet.cardinality();
	}


	/**
	 * Check that the parameters have been properly set for the
	 * operation to begin, and if not save an error message which
	 * can be obtained using getParameterErrorMessage.
	 */
	public boolean isReadyForExecution()
	{
		//
		// We must have a file bit set.
		//
		if (mFileBitSet == null)
			return setErrorAndReturnFalse("No checksum select list set");

		//
		// Allocate the message digest object.
		//
		try
		{
			mMessageDigest = MessageDigest.getInstance("MD5");
		}
		catch (Exception e)
		{
			return setErrorAndReturnFalse("Cannot initialise checksum: " + e);
		}

		//
		// We are ready for the checksum.
		// Say that we are ready.
		//
		setIsReadyForExecution();

		return true;
	}


	/**
	 * Run this operation.  This performs the checksum on the files set
	 * by the previously prepared arguments.  The checksums are stored in
	 * the FileInfo objects for future reference.
	 */
	public void runOperation()
	{
		//
		// Get the total count of files to checksum.
		//
		int fileCount = mManager.getFileCount();
		int processedCount = 0;
		int failedCount = 0;

		//
		// Tell the listener the number of files.
		//
		updateTotalFileCount(fileCount);

		//
		// Allocate new bit sets to hold the results.
		//
		mSuccessBitSet = new BitSet(fileCount);
		mFailureBitSet = new BitSet(fileCount);

		//
		// Iterate over all of the files in the file table.
		//
		for (int index = 0; index < fileCount; index++)
		{
			//
			// If this file is not in the bit set then continue.
			//
			if (!mFileBitSet.get(index))
				continue;

			//
			// Get the file handle for this index and let the
			// listener know about it.
			//
			FileInfo fileInfo = mManager.getFileInfo(index);

			updateFileName(fileInfo.getFileName());

			//
			// Perform the checksum and get the status.
			//
			boolean successFlag = checksum(fileInfo);

			//
			// If we are being cancelled then abort the operation now.
			//
			if (isCancelled())
				return;

			//
			// If the status is positive, then the file matched.
			// If the status is zero, the file is not matched.
			// If the status is negative, the file could not be read.
			//
			updateProcessedFileCount(++processedCount);

			if (successFlag)
				mSuccessBitSet.set(index);
			else
			{
				mFailureBitSet.set(index);
				updateFailedFileCount(++failedCount);
			}
		}

		//
		// The results are done.
		// Assign or update the select list bit sets if necessary.
		//
		if (mSuccessSelectList != null)
		{
			if (mIsSuccessSelectListCleared)
				mSuccessSelectList.setFileBitSet(mSuccessBitSet);
			else
				mSuccessSelectList.orOperation(mSuccessBitSet);
		}

		if (mFailureSelectList != null)
		{
			if (mIsFailureSelectListCleared)
				mFailureSelectList.setFileBitSet(mFailureBitSet);
			else
				mFailureSelectList.orOperation(mFailureBitSet);
		}
	}


	/**
	 * Perform a checksum on the indicated file or directory.
	 * The checksum is stored in the FileInfo for later reference.
	 * Returns true if the checksum was successful.
	 */
	protected boolean checksum(FileInfo fileInfo)
	{
		//
		// Get a file handle for the file.
		//
		File file = fileInfo.getFile();

		//
		// Do the checksum appropriately depending on whether the file
		// is a directory or not.
		//
		byte[] checksumBytes;

		if (file.isDirectory())
			checksumBytes = checksumDirectory(file);
		else
			checksumBytes = checksumRegularFile(file);

		//
		// Convert the checksum into a hex string.
		//
		String checksum = Utils.buildHexString(checksumBytes);

		//
		// Set the checksum into the file if we haven't been cancelled.
		//
		if (!isCancelled())
			fileInfo.setChecksum(checksum);

		//
		// Return the checksum status.
		//
		return (checksum != null);
	}


	/**
	 * Checksum a directory.
	 * This means perform a checksum of the sorted file names in the
	 * directory.  Returns null if the directory could not be read.
	 */
	protected byte[] checksumDirectory(File file)
	{
		//
		// Get the list of the file names in the directory.
		//
		String[] fileNames = file.list();

		if (fileNames == null)
			return null;

		//
		// Sort the file names so that the checksum is the same for
		// two directories containing the same files.
		//
		Arrays.sort(fileNames);

		//
		// Reset the message digest for another use.
		//
		mMessageDigest.reset();

		//
		// Allocate a two byte array containing null characters to
		// use to separate the file names in the checksum.
		//
		byte[] nullBytes = new byte[2];

		//
		// Iterate over all of the file names calculating the digest
		// over the names, with two null bytes between each name.
		//
		for (int index = 0; index < fileNames.length; index++)
		{
			if (index > 0)
				mMessageDigest.update(nullBytes);

			mMessageDigest.update(fileNames[index].getBytes());
		}

		//
		// Finish the checksum and return it.
		//
		return mMessageDigest.digest();
	}


	/**
	 * Checksum the data contained in a regular file.
	 * Return null if the file was unreadable.
	 */
	protected byte[] checksumRegularFile(File file)
	{
		FileInputStream fileInputStream = null;
		byte[] checksumBytes = null;

		//
		// Catch exceptions so we can close the file and also so
		// that we can report the failure status.
		//
		try
		{
			//
			// Try to open the file for reading.
			//
			fileInputStream = new FileInputStream(file);

			//
			// Reset the message digest for another use.
			//
			mMessageDigest.reset();

			//
			// Get a buffer to read the bytes into.
			//
			byte[] buffer = new byte[8192 * 16];

			//
			// Read all of the bytes of the file and checksum them.
			// Stop early if we are cancelled.
			//
			while (!isCancelled())
			{
				//
				// Try to read the next buffer of data from the file.
				//
				int count = fileInputStream.read(buffer);

				if (count < 0)
					break;

				//
				// Update the digest with this buffer's data.
				//
				mMessageDigest.update(buffer, 0, count);
			}

			//
			// Close the file.
			//
			fileInputStream.close();
			fileInputStream = null;

			//
			// Get the checksum bytes.
			//
			checksumBytes = mMessageDigest.digest();
		}
		catch (Exception e)
		{
			//
			// Clear the checksum.
			//
			checksumBytes = null;
		}
		finally
		{
			//
			// Safely close the file.
			//
			Utils.safeClose(fileInputStream);
		}

		//
		// Return the checksum bytes.
		//
		return checksumBytes;
	}
}
