/*
 * File Selection - A java utility to browse and manipulate sets of files.
 * Copyright (C) 2005  David I. Bell
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fileselection.utilities;

import java.util.ArrayList;


/**
 * A simple collection-like class which is both an array and a hash map.
 * This only holds objects which implement the ArrayAndMapObject interface.
 * This allows objects to be quickly indexed either by their key value or
 * by their index number.  It is guaranteed that the index number for an
 * object in the ArrayAndMap doesn't change.
 *
 * The hashing used can be case sensitive or not according to the setting
 * in our constructor.
 */
public class ArrayAndMap
{
	/**
	 * The case independent hash map used to look up an object by its
	 * key value.  The lookup can either be case sensitive or not according
	 * to how we are configured.
	 */
	protected CaseIndependentHashMap mHashMap;

	/**
	 * The array list used to look up an object by its index value.
	 */
	protected ArrayList mArrayList;


	/**
	 * Constructor which accepts the initial size of the map and array
	 * and which sets us as being case sensitive or not.
	 */
	public ArrayAndMap(int initialSize, boolean isCaseSensitive)
	{
		mHashMap = new CaseIndependentHashMap(initialSize, isCaseSensitive);
		mArrayList = new ArrayList(initialSize);
	}


	/**
	 * Put an object into the hash table given its key value,
	 * overwritting any object previously stored by that key.
	 * Returns true if the object was newly added.
	 */
	public synchronized boolean put(ArrayAndMapObject object)
	{
		//
		// Get the key value for this object.
		//
		Object key = object.getKey();

		//
		// See if we already have an object for this key.
		//
		ArrayAndMapObject oldObject = (ArrayAndMapObject) mHashMap.get(key);

		//
		// If an old object already exists then remove it and replace it
		// with this object using the same index value.
		//
		if (oldObject != null)
		{
			//
			// Get the index of the old object and clear it from the
			// old object so that it will no longer work for us.
			//
			int index = oldObject.getIndex();

			oldObject.setIndex(-1);

			//
			// Store the object in the hash table using its key and using
			// the old index.
			//
			mHashMap.put(key, object);
			mArrayList.set(index, object);

			//
			// Store the index in the new object so it knows where it is.
			//
			object.setIndex(index);

			//
			// The object was not new.
			//
			return false;
		}

		//
		// This is a new object.
		// Allocate it a new index which is the current size of the array.
		//
		int index = mArrayList.size();

		//
		// Add the object to the hash map and the array according to its
		// key value and index value.
		//
		mHashMap.put(key, object);
		mArrayList.add(index, object);

		//
		// Store the index in the object.
		//
		object.setIndex(index);

		//
		// The object was newly added.
		//
		return true;
	}


	/**
	 * Return whether an object exists in our map having the indicated key.
	 */
	public boolean exists(Object key)
	{
		return mHashMap.containsKey(key);
	}


	/**
	 * Return whether the specified object is contained within our array.
	 * This is not the same as the key value.
	 */
	public boolean contains(Object value)
	{
		return mArrayList.contains(value);
	}


	/**
	 * Find an object in our hash map using its key value.
	 */
	public ArrayAndMapObject get(Object key)
	{
		return (ArrayAndMapObject) mHashMap.get(key);
	}


	/**
	 * Find an object in our hash map using its index value.
	 */
	public ArrayAndMapObject get(int index)
	{
		return (ArrayAndMapObject) mArrayList.get(index);
	}


	/**
	 * Return the count of objects in the map.
	 */
	public int getCount()
	{
		return mArrayList.size();
	}


	/**
	 * Return the ArrayList of the objects.
	 */
	public ArrayList getArrayList()
	{
		return mArrayList;
	}
}
